unit ASFSampleData;

interface

uses
  SysUtils, Classes, asfClasses, asfModel, asfData, DB, ADODB;

type
  TMainDataModule = class(TDataModule)
    AppSecurity: TAppSecurity;
    AppSecurityFormController: TAppSecurityFormController;
    AppSecurityModel: TAppSecurityModel;
    AppSecurityDataProvider: TAppSecurityDataProvider;
    ADOConnection: TADOConnection;
    procedure ADOConnectionAfterConnect(Sender: TObject);
    procedure AppSecurityDataProviderAddLogData(Sender: TObject; ALogRecordID:
      Integer; AUserData: TObject);
    procedure AppSecurityDataProviderCreateTable(Sender: TObject; const ATableName:
      string; AFields: TFieldDefs; AIndexes: TIndexDefs);
    procedure AppSecurityDataProviderExecuteQuery(Sender: TObject; const Query:
      string);
    procedure AppSecurityDataProviderGetLogData(Sender: TObject; ALogRecordID:
      Integer; var AUserData: TObject);
    procedure AppSecurityDataProviderOpenQuery(Sender: TObject; const Query:
      string; var DataSet: TDataSet);
    procedure AppSecurityDataProviderOpenTable(Sender: TObject; const ATableName:
      string; var DataSet: TDataSet);
    procedure AppSecurityShowLogRecordData(Sender: TObject; ALogRecordID: Integer;
      ALogRecordType: Integer);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainDataModule: TMainDataModule;

implementation

uses asfADO, Graphics, ASFSampleViewImage;

{$R *.dfm}

procedure TMainDataModule.ADOConnectionAfterConnect(Sender: TObject);
begin
  AppSecurityDataProviderExecuteQuery(nil, 'SET DATEFORMAT dmy');
end;

procedure TMainDataModule.AppSecurityDataProviderCreateTable(Sender: TObject;
  const ATableName: string; AFields: TFieldDefs; AIndexes: TIndexDefs);
begin
  if ATableName = AppSecurityDataProvider.DBStructure.LogTable then
    AFields.Add('Bitmap', ftBlob);

  CreateADOTable(ADOConnection, ATableName, AFields, AIndexes);
end;

procedure TMainDataModule.AppSecurityDataProviderOpenTable(Sender: TObject;
  const ATableName: string; var DataSet: TDataSet);
var
  ATable: TADOTable;
begin
  ATable := TADOTable.Create(Self);
  try
    ATable.Connection := ADOConnection;
    ATable.TableName := ATableName;
    ATable.Open;
    DataSet := ATable;
  except
    ATable.Free; raise
  end;
end;

procedure TMainDataModule.AppSecurityDataProviderOpenQuery(Sender: TObject;
  const Query: string; var DataSet: TDataSet);
var
  AQuery: TADOQuery;
begin
  AQuery := TADOQuery.Create(Self);
  try
    AQuery.Connection := ADOConnection;
    AQuery.SQL.Text := Query;
    AQuery.Open;
    DataSet := AQuery;
  except
    AQuery.Free; raise
  end;
end;

procedure TMainDataModule.AppSecurityDataProviderExecuteQuery(Sender: TObject;
  const Query: string);
var
  AQuery: TADOQuery;
begin
  AQuery := TADOQuery.Create(Self);
  try
    AQuery.Connection := ADOConnection;
    AQuery.SQL.Text := Query;
    AQuery.ExecSQL;
  finally
    AQuery.Free;
  end;
end;

procedure TMainDataModule.AppSecurityDataProviderAddLogData(Sender: TObject;
  ALogRecordID: Integer; AUserData: TObject);
var
  DataSet: TDataSet;
  Stream: TStream;
begin
  if AUserData is TBitmap then
  begin
    DataSet := AppSecurityDataProvider.GetLogRecord(ALogRecordID);
    try
      DataSet.Edit;
      Stream := DataSet.CreateBlobStream(DataSet.FieldByName('Bitmap'), bmWrite);
      try
        TBitmap(AUserData).SaveToStream(Stream);
      finally
        Stream.Free;
      end;
      DataSet.Post;
    finally
      DataSet.Free;
    end;
  end;
end;

procedure TMainDataModule.AppSecurityDataProviderGetLogData(Sender: TObject;
  ALogRecordID: Integer; var AUserData: TObject);
var
  DataSet: TDataSet;
  Stream: TStream;
begin
  DataSet := AppSecurityDataProvider.GetLogRecord(ALogRecordID);
  try
    Stream := DataSet.CreateBlobStream(DataSet.FieldByName('Bitmap'), bmRead);
    try
      if Stream.Size > 0 then
      begin
        AUserData := TBitmap.Create;
        TBitmap(AUserData).LoadFromStream(Stream);
      end else
        AUserData := nil
    finally
      Stream.Free;
    end;
  finally
    DataSet.Free;
  end;
end;

procedure TMainDataModule.AppSecurityShowLogRecordData(Sender: TObject;
  ALogRecordID: Integer; ALogRecordType: Integer);
var
  AObject: TObject;
begin
  AObject := AppSecurityDataProvider.GetLogData(ALogRecordID);
  if Assigned(AObject) then
  begin
    fmImageViewer.Image.Picture.Bitmap := TBitmap(AObject);
    try
      fmImageViewer.ShowModal
    finally
      fmImageViewer.Image.Picture.Bitmap := nil; 
      AObject.Free
    end;
  end;
end;

end.
