unit ASFSampleDataBDE;

interface

uses
  SysUtils, Classes, DB, DBTables, asfData, asfModel, asfClasses;

type
  TBDEDataModule = class(TDataModule)
    AppSecurity: TAppSecurity;
    AppSecurityFormController: TAppSecurityFormController;
    AppSecurityModel: TAppSecurityModel;
    AppSecurityDataProvider: TAppSecurityDataProvider;
    Database: TDatabase;
    procedure AppSecurityDataProviderCreateTable(Sender: TObject; const ATableName:
      string; AFields: TFieldDefs; AIndexes: TIndexDefs);
    procedure AppSecurityDataProviderExecuteQuery(Sender: TObject; const Query:
      string);
    procedure AppSecurityDataProviderOpenQuery(Sender: TObject; const Query:
      string; var DataSet: TDataSet);
    procedure AppSecurityDataProviderOpenTable(Sender: TObject; const ATableName:
      string; var DataSet: TDataSet);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  BDEDataModule: TBDEDataModule;

implementation

{$R *.dfm}

procedure TBDEDataModule.AppSecurityDataProviderCreateTable(Sender: TObject;
  const ATableName: string; AFields: TFieldDefs; AIndexes: TIndexDefs);
begin
  with TTable.Create(Self) do
  try
    DatabaseName := Self.Database.DatabaseName;
    TableName := ATableName;
    FieldDefs.Assign(AFields);
    IndexDefs.Assign(AIndexes);
    if Exists then DeleteTable;
    CreateTable;
  finally
    Free
  end;
end;

procedure TBDEDataModule.AppSecurityDataProviderOpenTable(Sender: TObject;
  const ATableName: string; var DataSet: TDataSet);
begin
  DataSet := TTable.Create(Self);
  try
    with TTable(DataSet) do begin
      DatabaseName := Self.Database.DatabaseName;
      TableName := ATableName;
    end;
    DataSet.Open;
  except
    DataSet.Free; raise;
  end;
end;

procedure TBDEDataModule.AppSecurityDataProviderExecuteQuery(Sender: TObject;
  const Query: string);
begin
  with TQuery.Create(Self) do
  try
    DatabaseName := Self.Database.DatabaseName;
    SQL.Text := Query;
    ExecSQL;
  finally
    Free;
  end;
end;

procedure TBDEDataModule.AppSecurityDataProviderOpenQuery(Sender: TObject;
  const Query: string; var DataSet: TDataSet);
begin
  DataSet := TQuery.Create(Self);
  try
    with TQuery(DataSet) do begin
      DatabaseName := Self.Database.DatabaseName;
      SQL.Text := Query;
    end;
    DataSet.Open;
  except
    DataSet.Free; raise;
  end;
end;

end.
