unit asfAccessEditor;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, cxGraphics, cxCustomData, cxStyles, cxTL, cxTextEdit, cxDropDownEdit,
  cxInplaceContainer, ExtCtrls, cxControls, cxTLData, cxDBTL, cxMaskEdit,
  asfClasses, asfModel, ComCtrls, ImgList, cxEdit, StdCtrls;

type
  TfmAccessEditor = class(TForm)
    ModelTree: TcxTreeList;
    OperationColumn: TcxTreeListColumn;
    AccessKindColumn: TcxTreeListColumn;
    ImageList: TImageList;
    Panel1: TPanel;
    OkButton: TButton;
    CancelButton: TButton;
    procedure FormShow(Sender: TObject);
    procedure ModelTreeInitEdit(Sender, AItem: TObject; AEdit: TcxCustomEdit);
    procedure AccessKindColumnPropertiesChange(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Private declarations }
    FAppSecurity: TAppSecurity;
    FObjectID: Integer;
    FEditUser: Boolean;
    FUserName: String;
    FRoleID: Integer;
    procedure BuildTreeItems(AParentNode: TcxTreeListNode; AModelItem:
      TAppSecurityModelItem);
    procedure SetParentAccessKind(AParentNode: TcxTreeListNode; const AValue:
      String);
    procedure SetChildAccessKind(AParentNode: TcxTreeListNode; const AValue:
      String);
    procedure SaveAccessState;
    procedure LoadAccessState;
    procedure GetUserInfo;
  public
    procedure BuildTreeList;
  end;

function ShowAccessEditor(AnAppSecurity: TAppSecurity; EditUser: Boolean; ID:
  Integer): Boolean;

implementation

uses
  asfData;

{$R *.dfm}

function ShowAccessEditor(AnAppSecurity: TAppSecurity; EditUser: Boolean; ID:
  Integer): Boolean;
begin
  with TfmAccessEditor.Create(Application) do
  try
    FAppSecurity := AnAppSecurity;
    FObjectID := ID;
    FEditUser := EditUser;
    Result := ShowModal = mrOk;
  finally
    Free
  end;
end;

{ TfmAccessEditor }

procedure TfmAccessEditor.FormShow(Sender: TObject);
begin
  BuildTreeList;
  LoadAccessState;

  if FEditUser then GetUserInfo;
end;

procedure TfmAccessEditor.FormCloseQuery(Sender: TObject; var CanClose:
  Boolean);
begin
  if ModalResult = mrOK then SaveAccessState;
end;

procedure TfmAccessEditor.GetUserInfo;
var
  ALogin, APassword: String;
begin
  FAppSecurity.DataProvider.GetUserInfo(FObjectID, FUserName, ALogin, APassword, FRoleID);
  Caption := '   : ' + FUserName;
end;

{***************************************************************************** }
{                                                          }
{ **************************************************************************** }

procedure TfmAccessEditor.BuildTreeList;
var
  I: Integer;
begin
  ModelTree.BeginUpdate;
  try
    ModelTree.Clear;
    with FAppSecurity.Model do
      for I := 0 to ItemCount - 1 do BuildTreeItems(nil, Items[I]);
  finally
    ModelTree.EndUpdate;
  end;
end;

procedure TfmAccessEditor.BuildTreeItems(AParentNode: TcxTreeListNode;
  AModelItem: TAppSecurityModelItem);
var
  Node: TcxTreeListNode;
  I: Integer;
begin
  Node := ModelTree.AddChild(AParentNode);
  Node.Values[OperationColumn.ItemIndex] := AModelItem.Name;
  Node.ImageIndex := 0;
  Node.SelectedIndex := 0;
  Node.Data := AModelItem;

  for I := 0 to AModelItem.ChildItemCount - 1 do
    BuildTreeItems(Node, AModelItem.ChildItems[I]);
end;

{***************************************************************************** }
{                                                }
{ **************************************************************************** }

{{
      
}
procedure TfmAccessEditor.ModelTreeInitEdit(Sender, AItem: TObject; AEdit:
  TcxCustomEdit);
begin
  (AEdit as TcxComboBox).Properties.Items :=
    TAppSecurityModelItem(ModelTree.FocusedNode.Data).AccessKinds
end;

{{
          
   (     ).

          (?).
}
procedure TfmAccessEditor.AccessKindColumnPropertiesChange(Sender: TObject);
begin
  ModelTree.InplaceEditor.PostEditValue;
  SetParentAccessKind(ModelTree.FocusedNode, '');
  SetChildAccessKind(ModelTree.FocusedNode, AccessKindColumn.EditValue);
end;

procedure TfmAccessEditor.SetParentAccessKind(AParentNode: TcxTreeListNode;
  const AValue: String);
begin
  while AParentNode <> nil do
  begin
    AParentNode := AParentNode.Parent;
    if AParentNode <> nil then AParentNode.Values[AccessKindColumn.ItemIndex] := AValue;
  end;
end;

procedure TfmAccessEditor.SetChildAccessKind(AParentNode: TcxTreeListNode;
  const AValue: String);
var
  I: Integer;
begin
  for I := 0 to AParentNode.Count - 1 do
  begin
    if TAppSecurityModelItem(AParentNode.Items[I].Data).AccessKinds.IndexOf(AValue) >= 0 then
      AParentNode.Items[I].Values[AccessKindColumn.ItemIndex] := AValue;

    SetChildAccessKind(AParentNode.Items[I], AValue);
  end;
end;

{***************************************************************************** }
{                    /                       }
{ **************************************************************************** }

procedure TfmAccessEditor.SaveAccessState;
var
  AObjectType: Integer;
  I: Integer;
begin
  if FEditUser then
    AObjectType := UserObjectType
  else
    AObjectType := RoleObjectType;

  FAppSecurity.DataProvider.ClearObjectRights(FObjectID, AObjectType);
  for I := 0 to ModelTree.Nodes.Count - 1 do
  if VarToStr(ModelTree.Nodes[I].Values[AccessKindColumn.ItemIndex]) <> '' then
    FAppSecurity.DataProvider.SetObjectRights(FObjectID, AObjectType,
    TAppSecurityModelItem(ModelTree.Nodes[I].Data).ID,
      ModelTree.Nodes[I].Values[AccessKindColumn.ItemIndex]);

  with FAppSecurity do
    if FEditUser and LogOptions.RegisterUserAccessChange then
      AddLogRecord(lmUserAccessChange, LogOptions.UserAccessChangeMessage + FUserName);
end;

{{
     .    
   ,      ,   
  ( ).
}
procedure TfmAccessEditor.LoadAccessState;

  procedure LoadData(ObjectID, ObjectType: Integer);
  var
    AccessKind: String;
    I: Integer;
  begin
    for I := 0 to ModelTree.Nodes.Count - 1 do
      if FAppSecurity.DataProvider.GetObjectRights(ObjectID, ObjectType,
        TAppSecurityModelItem(ModelTree.Nodes[I].Data).ID, AccessKind)
      then
        ModelTree.Nodes[I].Values[AccessKindColumn.ItemIndex] := AccessKind;
  end;

begin
  if FEditUser then
  begin
    if FRoleID <> 0 then LoadData(FRoleID, RoleObjectType);
    LoadData(FObjectID, UserObjectType);
  end
  else
    LoadData(FObjectID, RoleObjectType);

  ModelTree.FullExpand
end;

end.
