unit asfAddLinkWizard;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls, asfClasses, AppEvnts, asfModel, ImgList;

type
  TfmComponentSelector = class(TForm)
    ComponentTree: TTreeView;
    Panel: TPanel;
    OkButton: TButton;
    CancelButton: TButton;
    Panel1: TPanel;
    edFormName: TEdit;
    Label1: TLabel;
    cbComponentType: TComboBox;
    Label2: TLabel;
    ApplicationEvents: TApplicationEvents;
    ImageList: TImageList;
    procedure ApplicationEventsIdle(Sender: TObject; var Done: Boolean);
    procedure FormShow(Sender: TObject);
    procedure UpdateComponentList(Sender: TObject);
    procedure cbComponentTypeKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ComponentTreeDblClick(Sender: TObject);
  private
    procedure FillComponentTree;
    function GetSelectedComponent: TComponent;
    { Private declarations }
  public
    { Public declarations }
  end;

function ShowAddLinkWizard(SecurityItem: TAppSecurityModelItem; FormController:
  TAppSecurityFormController): Boolean;

implementation

uses asfActions;

{$R *.dfm}

procedure CreateAppSecurityActionLink(FormController:
  TAppSecurityFormController; SecurityItem: TAppSecurityModelItem; Action:
  TAppSecurityAction);
var
  NewItem: TAppSecurityFormControllerItem;
begin
  NewItem := FormController.AddItem;
  NewItem.FormName := Action.Owner.ClassName;
  NewItem.ModelItemID := SecurityItem.ID;
  NewItem.ComponentName := Action.Name;
  NewItem.AccessState := SecurityItem.AccessKinds[0];
  NewItem.PropertyName := 'Disabled';
  NewItem.PropertyValue := False;

  NewItem := FormController.AddItem;
  NewItem.FormName := Action.Owner.ClassName;
  NewItem.ModelItemID := SecurityItem.ID;
  NewItem.ComponentName := Action.Name;
  NewItem.AccessState := SecurityItem.AccessKinds[1];
  NewItem.PropertyName := 'Disabled';
  NewItem.PropertyValue := True;
end;

procedure CreateAppSecurityAccessItemLink(FormController:
  TAppSecurityFormController; SecurityItem: TAppSecurityModelItem; AccessItem:
  TAppSecurityFormAccessItem);
var
  NewItem: TAppSecurityFormControllerItem;
begin
  NewItem := FormController.AddItem;
  NewItem.FormName := AccessItem.Owner.ClassName;
  NewItem.ModelItemID := SecurityItem.ID;
  NewItem.ComponentName := AccessItem.Name;
  NewItem.AccessState := SecurityItem.AccessKinds[0];
  NewItem.PropertyName := 'Enabled';
  NewItem.PropertyValue := True;

  NewItem := FormController.AddItem;
  NewItem.FormName := AccessItem.Owner.ClassName;
  NewItem.ModelItemID := SecurityItem.ID;
  NewItem.ComponentName := AccessItem.Name;
  NewItem.AccessState := SecurityItem.AccessKinds[1];
  NewItem.PropertyName := 'Enabled';
  NewItem.PropertyValue := False;
end;

procedure CreateComponentLinks(FormController: TAppSecurityFormController;
  SecurityItem: TAppSecurityModelItem; Component: TComponent);
begin
  { TODO :        }
  if Component is TAppSecurityAction then
    CreateAppSecurityActionLink(FormController, SecurityItem, TAppSecurityAction(Component)) else
  if Component is TAppSecurityFormAccessItem then
    CreateAppSecurityAccessItemLink(FormController, SecurityItem, TAppSecurityFormAccessItem(Component)) else
end;

function CanCreateComponentLink(Component: TComponent): Boolean;
begin
  { TODO :        }
  Result := (Component is TAppSecurityAction) or (Component is TAppSecurityFormAccessItem)
end;

function ShowAddLinkWizard(SecurityItem: TAppSecurityModelItem; FormController:
  TAppSecurityFormController): Boolean;
begin
  with TfmComponentSelector.Create(Application) do
  try
    Result := ShowModal = mrOk;
    if Result then CreateComponentLinks(FormController, SecurityItem, GetSelectedComponent);
  finally
    Free
  end;
end;

{ TfmComponentSelector }

procedure TfmComponentSelector.FormShow(Sender: TObject);
begin
  FillComponentTree;
end;

procedure TfmComponentSelector.UpdateComponentList(Sender: TObject);
begin
  FillComponentTree
end;

procedure TfmComponentSelector.FillComponentTree;

  {{
     ,  
  }
  procedure AddComponentItems(TreeNode: TTreeNode; AOwner: TComponent);
  var
    I: Integer;
    Node: TTreeNode;
  begin
    with AOwner do
      for I := 0 to ComponentCount - 1 do
      if CanCreateComponentLink(Components[I]) then
      begin
        if (cbComponentType.ItemIndex < 0) or (Pos(cbComponentType.Text, Components[I].ClassName) > 0) then
        begin
          Node := ComponentTree.Items.AddChild(TreeNode, Components[I].Name);
          Node.ImageIndex := 1;
          Node.SelectedIndex := 1;
          Node.Data := Components[I];
        end;

        if cbComponentType.Items.IndexOf(Components[I].ClassName) < 0 then
          cbComponentType.Items.Add(Components[I].ClassName);
      end
  end;

var
  I: Integer;
  Node: TTreeNode;
begin
  with ComponentTree do
  begin
    Items.BeginUpdate;
    try
      Items.Clear;
      with Screen do
        for I := 0 to FormCount - 1 do
          if (edFormName.Text = '') or (Pos(edFormName.Text, Forms[I].ClassName) > 0) then
          begin
            Node := Items.Add(nil, Forms[I].ClassName);
            Node.ImageIndex := 0;
            Node.SelectedIndex := 0;
            AddComponentItems(Node, Forms[I]);
            {    ,  ,  
                }
            if Node.Count = 0 then Node.Delete;
          end;
    finally
      Items.EndUpdate;
    end;
  end
end;

procedure TfmComponentSelector.cbComponentTypeKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if Key = VK_DELETE then
  begin
    cbComponentType.ItemIndex := -1;
    Key := 0;
  end;
end;

procedure TfmComponentSelector.ApplicationEventsIdle(Sender: TObject; var Done:
  Boolean);
begin
  OkButton.Enabled := Assigned(ComponentTree.Selected) and Assigned(ComponentTree.Selected.Data);
end;

procedure TfmComponentSelector.ComponentTreeDblClick(Sender: TObject);
begin
  if OkButton.Enabled then ModalResult := mrOK
end;

function TfmComponentSelector.GetSelectedComponent: TComponent;
begin
  Result := TComponent(ComponentTree.Selected.Data);
end;

end.
