unit asfBaseUserEditor;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, 
  Dialogs, StdCtrls, ComCtrls, ActnList, asfClasses, cxGraphics, cxCustomData,
  cxStyles, cxTL, cxMaskEdit, cxInplaceContainer, cxDBTL, cxControls, cxTLData,
  ImgList, DB, asfActions, cxLookAndFeels, cxLookAndFeelPainters;

type
  {{
        (  ).
   :
        -   
        -    
        -   
        -   
        -    
        -    
        -      
  }
  TfrmBaseUserEditor = class(TFrame)
    AddButton: TButton;
    AddGroupButton: TButton;
    DeleteButton: TButton;
    ActionList: TActionList;
    EditButton: TButton;
    AddAction: TAppSecurityAction;
    AddGroupAction: TAppSecurityAction;
    DeleteAction: TAppSecurityAction;
    EditAction: TAppSecurityAction;
    EditAccessButton: TButton;
    EditAccessAction: TAppSecurityAction;
    ImageList: TImageList;
    TreeList: TcxDBTreeList;
    NameColumn: TcxDBTreeListColumn;
    DataSource: TDataSource;
    procedure ActionListUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure AddGroupActionExecute(Sender: TObject);
    procedure DeleteActionExecute(Sender: TObject);
    procedure EditAccessActionExecute(Sender: TObject);
    procedure EditActionExecute(Sender: TObject);
    procedure TreeListDblClick(Sender: TObject);
    procedure TreeListDragOver(Sender, Source: TObject; X, Y: Integer; State:
      TDragState; var Accept: Boolean);
  private
    { Private declarations }
    FNameField: String;
    FIdField: String;
    FParentIdField: String;
    FObjectTypeField: String;
    FGroupObjectType: Integer;
    FAppSecurity: TAppSecurity;
    procedure SetAppSecurity(Value: TAppSecurity);
  protected
    procedure ShowData(ADataSet: TDataSet);
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure SetupForm; virtual; abstract;
    procedure AddRecord(ObjectName: String; ObjectType: Integer);
    function EditObject(ObjectID: Integer): Boolean; virtual; abstract;
    procedure EditObjectAccess(ObjectID: Integer); virtual; abstract;
    procedure RenameObject(const AObjectName: String);
    procedure RefreshDataSet(AObjectToSelect: Integer);
  public
    destructor Destroy; override;
    { Public declarations }
    property IdField: String read FIdField write FIdField;
    property ParentIdField: String read FParentIdField write FParentIdField;
    property NameField: String read FNameField write FNameField;
    property ObjectTypeField: String read FObjectTypeField write FObjectTypeField;
    property GroupObjectType: Integer read FGroupObjectType write FGroupObjectType;
  published
    property AppSecurity: TAppSecurity read FAppSecurity write SetAppSecurity;
  end;

implementation

{$R *.dfm}

destructor TfrmBaseUserEditor.Destroy;
begin
  DataSource.DataSet.Free;
  inherited Destroy;
end;

procedure TfrmBaseUserEditor.Notification(AComponent: TComponent; Operation:
  TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = AppSecurity) and (Operation = opRemove) then AppSecurity := nil
end;

procedure TfrmBaseUserEditor.SetAppSecurity(Value: TAppSecurity);
begin
  if FAppSecurity <> Value then
  begin
    FAppSecurity := Value;
    if Value <> nil then Value.FreeNotification(Self);
    SetupForm;
  end;
end;

procedure TfrmBaseUserEditor.ActionListUpdate(Action: TBasicAction; var
  Handled: Boolean);
begin
  DeleteAction.Enabled := Assigned(TreeList.FocusedNode);
  EditAction.Enabled := DeleteAction.Enabled;
  EditAccessAction.Enabled := DeleteAction.Enabled and
    (DataSource.DataSet.FieldByName(ObjectTypeField).AsInteger <> GroupObjectType)
end;

procedure TfrmBaseUserEditor.AddGroupActionExecute(Sender: TObject);
var
  GroupName: String;
begin
  if InputQuery(' ', ' ', GroupName) then
    AddRecord(GroupName, GroupObjectType);
end;

procedure TfrmBaseUserEditor.DeleteActionExecute(Sender: TObject);
begin
  TreeList.DeleteSelection
end;

procedure TfrmBaseUserEditor.EditActionExecute(Sender: TObject);
var
  GroupName: String;
  ObjectID: Integer;
begin
  if DataSource.DataSet.FieldByName(ObjectTypeField).AsInteger = GroupObjectType then
  begin
    GroupName := DataSource.DataSet.FieldByName(NameField).AsString;
    if InputQuery(' ', ' ', GroupName) then
      RenameObject(GroupName)
  end else
  begin
    ObjectID := DataSource.DataSet.FieldByName(IdField).AsInteger;
    if EditObject(ObjectID) then RefreshDataSet(ObjectID);
  end;
end;

procedure TfrmBaseUserEditor.EditAccessActionExecute(Sender: TObject);
begin
  EditObjectAccess(DataSource.DataSet.FieldByName(IdField).AsInteger)
end;

procedure TfrmBaseUserEditor.TreeListDragOver(Sender, Source: TObject; X, Y:
  Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := Source = TreeList;
end;

procedure TfrmBaseUserEditor.TreeListDblClick(Sender: TObject);
begin
  if DataSource.DataSet.FieldByName(ObjectTypeField).AsInteger = GroupObjectType then
    EditAction.Execute
  else
    EditAccessAction.Execute
end;

{***************************************************************************** }
{                                                                              }
{ **************************************************************************** }

procedure TfrmBaseUserEditor.ShowData(ADataSet: TDataSet);
begin
  NameColumn.DataBinding.FieldName := NameField;
  TreeList.DataController.KeyField := IdField;
  TreeList.DataController.ParentField := ParentIdField;
  TreeList.DataController.ImageIndexFieldName := ObjectTypeField;
  DataSource.DataSet := ADataSet;
end;

procedure TfrmBaseUserEditor.AddRecord(ObjectName: String; ObjectType: Integer);
begin
  with DataSource.DataSet do
  begin
    Append;
    FieldByName(NameField).AsString := ObjectName;
    FieldByName(ObjectTypeField).AsInteger := ObjectType;
    Post;
  end;
end;

procedure TfrmBaseUserEditor.RenameObject(const AObjectName: String);
begin
  with DataSource.DataSet do
  begin
    Edit;
    FieldByName(NameField).AsString := AObjectName;
    Post;
  end;
end;

procedure TfrmBaseUserEditor.RefreshDataSet(AObjectToSelect: Integer);
begin
  DataSource.DataSet.Close;
  DataSource.DataSet.Open;
  DataSource.DataSet.Locate(IdField, AObjectToSelect, []);
end;

end.
