unit asfFormControllerEditor;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, asfModelEditor, ComCtrls, ExtCtrls, ImgList, asfModelTreeView,
  asfClasses, asfModel, ActnList, StdCtrls, ToolWin;

type
  TfmFormControllerlEditor = class(TForm)
    Splitter1: TSplitter;
    Splitter2: TSplitter;
    ModelTreeView: TAppSecurityModelTreeView;
    Panel3: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    edFormName: TComboBox;
    cbComponentName: TComboBox;
    cbPropertyName: TComboBox;
    cbAccessKind: TComboBox;
    cbValue: TComboBox;
    edModelItemName: TEdit;
    ItemList: TListView;
    OkButton: TButton;
    CancelButton: TButton;
    Bevel1: TBevel;
    ImageList: TImageList;
    ActionList: TActionList;
    AddItemAction: TAction;
    DeleteItemAction: TAction;
    FormControllerStorage: TAppSecurityFormController;
    ToolbarImages: TImageList;
    ToolBar: TToolBar;
    ToolButton1: TToolButton;
    ToolButton4: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    DeleteAllAction: TAction;
    DisabledImages: TImageList;
    ToolButton5: TToolButton;
    CopyItemAction: TAction;
    ToolButton6: TToolButton;
    AddItemWizardAction: TAction;
    FilterButton: TToolButton;
    ToolButton8: TToolButton;
    procedure ActionListUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure AddItemActionExecute(Sender: TObject);
    procedure cbComponentNameExit(Sender: TObject);
    procedure cbPropertyNameExit(Sender: TObject);
    procedure CopyItemActionExecute(Sender: TObject);
    procedure DeleteAllActionExecute(Sender: TObject);
    procedure DeleteItemActionExecute(Sender: TObject);
    procedure edFormNameExit(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormShow(Sender: TObject);
    procedure ItemListChange(Sender: TObject; Item: TListItem; Change: TItemChange);
    procedure AddItemWizardActionExecute(Sender: TObject);
    procedure ItemPropertiesChange(Sender: TObject);
    procedure FilterButtonClick(Sender: TObject);
    procedure ModelTreeViewClick(Sender: TObject);
  private
    FForm: TComponent;
    FChanging: Boolean;
    procedure ShowItems;
    procedure ShowItemInList(Item: TAppSecurityFormControllerItem; ListItem:
      TListItem);
    procedure ShowItemProperties(AItem: TAppSecurityFormControllerItem);
    function GetModelItem(SecurityItemID: Integer): TAppSecurityModelItem;
    procedure FillValueList;
    procedure UpdateItemProperties(AItem: TAppSecurityFormControllerItem);
    procedure FillFormList;
    function GetFormByName(AFormName: String): TComponent;
    { Private declarations }
  protected
    procedure FillComponentList;
    procedure FillPropertyList;
  public
    { Public declarations }
  end;

function ShowFormControllerEditor(AFormController:
  TAppSecurityFormController; AForm: TComponent): Boolean;

implementation

uses
  asfUtils, asfAddLinkWizard;

{$R *.dfm}

function ShowFormControllerEditor(AFormController:
  TAppSecurityFormController; AForm: TComponent): Boolean;
begin
  with TfmFormControllerlEditor.Create(Application) do
  try
   if Not Assigned(AFormController.AppSecurity) or
      Not Assigned(AFormController.AppSecurity.Model)
   then raise Exception.Create('    ' + AFormController.Name);

   FormControllerStorage.Assign(AFormController);
   FForm := AForm;
   ModelTreeView.Model := AFormController.AppSecurity.Model;
   Result := ShowModal = mrOk;
   if Result then
     AFormController.Assign(FormControllerStorage);
  finally
    Free
  end;
end;

{ TfmFormControllerlEditor }

procedure TfmFormControllerlEditor.FormShow(Sender: TObject);
begin
  inherited;
  ModelTreeView.FullExpand;
  ShowItems;
  FillFormList;
end;

function TfmFormControllerlEditor.GetModelItem(SecurityItemID: Integer):
  TAppSecurityModelItem;
begin
  Result := ModelTreeView.Model.GetItemByID(SecurityItemID)
end;

function TfmFormControllerlEditor.GetFormByName(AFormName: String): TComponent;
var
  AOwner: TComponent;
  I: Integer;
begin
  if Assigned(FForm) and (UpperCase(FForm.ClassName) = UpperCase(AFormName)) then
    Result := FForm
  else begin
    for I := 0 to Screen.FormCount - 1 do
      if UpperCase(Screen.Forms[I].ClassName) = UpperCase(AFormName) then
      begin
        Result := Screen.Forms[I];
        Exit;
      end;
    Result := nil;
  end;
end;

{***************************************************************************** }
{                                                    }
{ **************************************************************************** }

procedure TfmFormControllerlEditor.ModelTreeViewClick(Sender: TObject);
begin
  if FilterButton.Down then ShowItems
end;

procedure TfmFormControllerlEditor.FilterButtonClick(Sender: TObject);
begin
  ShowItems;
end;

procedure TfmFormControllerlEditor.ShowItems;
var
  I: Integer;
begin
  ItemList.Items.BeginUpdate;
  FChanging := True;
  try
    ItemList.Items.Clear;
    with FormControllerStorage do
      for I := 0 to ItemCount - 1 do
        if Not FilterButton.Down or Not Assigned(ModelTreeView.Selected) or
          (Items[I].ModelItemID = ModelTreeView.GetModelItemFromNode(ModelTreeView.Selected).ID)
        then
          ShowItemInList(Items[I], ItemList.Items.Add)
  finally
    FChanging := False;
    ItemList.Items.EndUpdate;
  end;
end;

procedure TfmFormControllerlEditor.ShowItemInList(Item:
  TAppSecurityFormControllerItem; ListItem: TListItem);
begin
  ListItem.Caption := GetModelItem(Item.ModelItemID).Name;
  ListItem.SubItems.Clear;
  ListItem.SubItems.Add(Item.AccessState);
  ListItem.SubItems.Add(Item.FormName);
  ListItem.SubItems.Add(Item.ComponentName);
  ListItem.SubItems.Add(Item.PropertyName);
  ListItem.SubItems.Add(VarToStr(Item.PropertyValue));
  ListItem.Data := Item;
end;

procedure TfmFormControllerlEditor.ActionListUpdate(Action: TBasicAction; var
  Handled: Boolean);
begin
  AddItemAction.Enabled := ModelTreeView.Selected <> nil;
  DeleteItemAction.Enabled := ItemList.Selected <> nil;
  CopyItemAction.Enabled := DeleteItemAction.Enabled;
end;

procedure TfmFormControllerlEditor.AddItemActionExecute(Sender: TObject);
var
  NewItem: TAppSecurityFormControllerItem;
begin
  FChanging := True;
  try
    NewItem := FormControllerStorage.AddItem;
    NewItem.FormName := FForm.ClassName;
    NewItem.ModelItemID := ModelTreeView.GetModelItemFromNode(ModelTreeView.Selected).ID;
    ShowItemInList(NewItem, ItemList.Items.Add);
  finally
    FChanging := False;
  end;
end;

procedure TfmFormControllerlEditor.AddItemWizardActionExecute(Sender: TObject);
begin
  if Not Assigned(ModelTreeView.Selected) then
    Application.MessageBox('   ', '', MB_ICONERROR or MB_OK)
  else
    if ShowAddLinkWizard(ModelTreeView.GetModelItemFromNode(ModelTreeView.Selected), FormControllerStorage) then ShowItems;
end;

procedure TfmFormControllerlEditor.CopyItemActionExecute(Sender: TObject);
var
  NewItem: TAppSecurityFormControllerItem;
  SourceItem: TAppSecurityFormControllerItem;
begin
  FChanging := True;
  try
    SourceItem := TAppSecurityFormControllerItem(ItemList.Selected.Data);
    NewItem := FormControllerStorage.AddItem;
    NewItem.Assign(SourceItem);
    ShowItemInList(NewItem, ItemList.Items.Add);
  finally
    FChanging := False;
  end;
end;

procedure TfmFormControllerlEditor.DeleteItemActionExecute(Sender: TObject);
begin
  FChanging := True;
  try
    FormControllerStorage.DeleteItem(TAppSecurityFormControllerItem(ItemList.Selected.Data));
    ItemList.Selected.Delete;
    ShowItemProperties(nil);
  finally
    FChanging := False;
  end;
end;

procedure TfmFormControllerlEditor.DeleteAllActionExecute(Sender: TObject);
begin
  if Application.MessageBox('  ?', '',
    MB_ICONQUESTION or MB_YESNO) = idYes
  then begin
    ItemList.Clear;
    FormControllerStorage.Clear;
    ShowItemProperties(nil);
  end;
end;

{***************************************************************************** }
{                     /             }
{ **************************************************************************** }

procedure TfmFormControllerlEditor.ItemListChange(Sender: TObject; Item:
  TListItem; Change: TItemChange);
begin
  if Not FChanging then
  begin
    if Item <> nil then
      ShowItemProperties(TAppSecurityFormControllerItem(Item.Data))
    else
      ShowItemProperties(nil)
  end;
end;

procedure TfmFormControllerlEditor.ShowItemProperties(AItem:
  TAppSecurityFormControllerItem);
var
  AModelItem: TAppSecurityModelItem;
begin
  if AItem <> nil then
  begin
    AModelItem := GetModelItem(AItem.ModelItemID);
    edModelItemName.Text := AModelItem.Name;
    cbAccessKind.Items := AModelItem.AccessKinds;
    cbAccessKind.ItemIndex := cbAccessKind.Items.IndexOf(AItem.AccessState);
    edFormName.Text := AItem.FormName;
    FillComponentList;
    cbComponentName.Text := AItem.ComponentName;
    FillPropertyList;
    cbPropertyName.Text := AItem.PropertyName;
    FillValueList;
    cbValue.Text := AItem.PropertyValue;
  end else
  begin
    edModelItemName.Text := '';
    cbAccessKind.Items.Clear;
    cbAccessKind.ItemIndex := -1;
    edFormName.Text := '';
    cbComponentName.Items.Clear;
    cbComponentName.Text := '';
    cbPropertyName.Items.Clear;
    cbPropertyName.Text := '';
    cbValue.Items.Clear;
    cbValue.Text := '';
  end;
end;

procedure TfmFormControllerlEditor.UpdateItemProperties(AItem:
  TAppSecurityFormControllerItem);
begin
  AItem.AccessState := cbAccessKind.Text;
  AItem.FormName := edFormName.Text;
  AItem.ComponentName := cbComponentName.Text;
  AItem.PropertyName := cbPropertyName.Text;
  AItem.PropertyValue := cbValue.Text;
end;

procedure TfmFormControllerlEditor.FormCloseQuery(Sender: TObject; var
  CanClose: Boolean);
begin
  if (ModalResult = mrOk) and (ItemList.Selected <> nil) then
    UpdateItemProperties(TAppSecurityFormControllerItem(ItemList.Selected.Data));
end;

{***************************************************************************** }
{                                                 }
{ **************************************************************************** }

procedure TfmFormControllerlEditor.edFormNameExit(Sender: TObject);
begin
  FillComponentList;
end;

procedure TfmFormControllerlEditor.cbComponentNameExit(Sender: TObject);
begin
  FillPropertyList;
end;

procedure TfmFormControllerlEditor.cbPropertyNameExit(Sender: TObject);
begin
  FillValueList;
end;

{{
  
}
procedure TfmFormControllerlEditor.FillComponentList;
var
  AOwner: TComponent;
  I: Integer;
begin
  AOwner := GetFormByName(edFormName.Text);
  if AOwner <> nil then
    with cbComponentName do
    begin
      Items.Clear;
      with AOwner do
        for I := 0 to ComponentCount - 1 do
          Items.Add(Components[I].Name);
    end
  else
    cbComponentName.Items.Clear;
end;

{{
   
}
procedure TfmFormControllerlEditor.FillPropertyList;
var
  AComponent: TComponent;
  AOwner: TComponent;
  I: Integer;
begin
  AOwner := GetFormByName(edFormName.Text);
  if Assigned(AOwner) and Assigned(AOwner.FindComponent(cbComponentName.Text)) then
    with cbPropertyName do
    begin
      Items.Clear;
      AComponent := AOwner.FindComponent(cbComponentName.Text);
      GetObjectProperties(AComponent, Items);
    end
  else
    cbPropertyName.Items.Clear;
end;

{{
  
}
procedure TfmFormControllerlEditor.FillValueList;
var
  AComponent: TComponent;
  AOwner: TComponent;
begin
  AOwner := GetFormByName(edFormName.Text);
  if Assigned(AOwner) and Assigned(AOwner.FindComponent(cbComponentName.Text)) then
    with cbValue do
    begin
      Items.Clear;
      AComponent := AOwner.FindComponent(cbComponentName.Text);
      GetEnumerationPropertyValues(AComponent, cbPropertyName.Text, Items);
    end
  else
    cbValue.Items.Clear;
end;

{{
  
}
procedure TfmFormControllerlEditor.FillFormList;
var
  I: Integer;
begin
  with edFormName do
  begin
    Items.Clear;
    with Screen do
      for I := 0 to FormCount - 1 do
        Items.Add(Forms[I].ClassName);
  end
end;

procedure TfmFormControllerlEditor.ItemPropertiesChange(Sender: TObject);
var
  Item: TListItem;
begin
  Item := ItemList.Selected;
  if Not FChanging and (Item <> nil) then
  begin
    UpdateItemProperties(TAppSecurityFormControllerItem(Item.Data));
    ItemList.Items.BeginUpdate;
    try
      ShowItemInList(TAppSecurityFormControllerItem(Item.Data), ItemList.Selected);
    finally
      ItemList.Items.EndUpdate;
    end;
  end;
end;

end.
