unit asfModelEditor;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, asfModel, ComCtrls, asfModelTreeView, ImgList, StdCtrls, ExtCtrls,
  ToolWin, ActnList;

type
  TfmModelEditor = class(TForm)
    ModelTreeView: TAppSecurityModelTreeView;
    ImageList: TImageList;
    Panel1: TPanel;
    OkButton: TButton;
    CancelButton: TButton;
    ToolBar: TToolBar;
    ToolButton1: TToolButton;
    mmStates: TMemo;
    ToolbarImages: TImageList;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ActionList: TActionList;
    AddAction: TAction;
    DeleteAction: TAction;
    ClearAction: TAction;
    Splitter: TSplitter;
    AppSecurityModel: TAppSecurityModel;
    EditAction: TAction;
    ToolButton5: TToolButton;
    procedure EditActionExecute(Sender: TObject);
    procedure AddActionExecute(Sender: TObject);
    procedure ClearActionExecute(Sender: TObject);
    procedure DeleteActionExecute(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure mmStatesChange(Sender: TObject);
    procedure ModelTreeViewChange(Sender: TObject; Node: TTreeNode);
  private
    FUpdatingStates: Boolean;
    { Private declarations }
  public
    { Public declarations }
  end;

function ShowModelEditor(AModel: TAppSecurityModel): Boolean;

implementation

{$R *.dfm}

function ShowModelEditor(AModel: TAppSecurityModel): Boolean;
begin
  with TfmModelEditor.Create(Application) do
  begin
    AppSecurityModel.Assign(AModel);
    Result := ShowModal = mrOK;
    if Result then
      AModel.Assign(AppSecurityModel);
  end;
end;

{ TfmModelEditor }

procedure TfmModelEditor.FormShow(Sender: TObject);
begin
  ModelTreeView.RefreshTreeView;
  ModelTreeView.FullExpand
end;

procedure TfmModelEditor.AddActionExecute(Sender: TObject);
begin
  ModelTreeView.InsertNewItem;
end;

procedure TfmModelEditor.DeleteActionExecute(Sender: TObject);
begin
  ModelTreeView.DeleteCurrentItem;
  ModelTreeViewChange(nil, nil);
end;

procedure TfmModelEditor.EditActionExecute(Sender: TObject);
begin
  ModelTreeView.Selected.EditText 
end;

procedure TfmModelEditor.ClearActionExecute(Sender: TObject);
begin
  if Application.MessageBox('  ?', '',
    MB_ICONQUESTION or MB_YESNO) = idYes
  then begin
    ModelTreeView.Clear;
    ModelTreeViewChange(nil, nil);
  end;
end;

procedure TfmModelEditor.ModelTreeViewChange(Sender: TObject; Node: TTreeNode);
begin
  if Assigned(ModelTreeView.Selected) then
  begin
    FUpdatingStates := True;
    try
      mmStates.Lines := ModelTreeView.GetModelItemFromNode(ModelTreeView.Selected).AccessKinds;
    finally
      FUpdatingStates := False;
    end;
    mmStates.ReadOnly := False;
  end
  else
  begin
    mmStates.Lines.Clear;
    mmStates.ReadOnly := True;
  end;
end;

procedure TfmModelEditor.mmStatesChange(Sender: TObject);
begin
  if Assigned(ModelTreeView.Selected) and Not FUpdatingStates then
    ModelTreeView.GetModelItemFromNode(ModelTreeView.Selected).AccessKinds := mmStates.Lines
end;

end.
