unit asfUserEditor;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, asfClasses;

type
  TfmUserEditor = class(TForm)
    edUserName: TEdit;
    Label1: TLabel;
    edLogin: TEdit;
    edPassword: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    OkButton: TButton;
    CancelButton: TButton;
    Bevel1: TBevel;
    cbRole: TComboBox;
    Label4: TLabel;
    Image1: TImage;
    procedure cbRoleKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormShow(Sender: TObject);
  private
    FUserID: Integer;
    FAppSecurity: TAppSecurity;
    procedure LoadData(AUserID: Integer);
    function SaveData(AUserID: Integer): Boolean;
    procedure LoadRoles;
    function IsValidLogin: Boolean;
    { Private declarations }
  public
    { Public declarations }
  end;

function EditUser(AppSecurity: TAppSecurity; AUserID: Integer): Boolean;

implementation

{$R *.dfm}

function EditUser(AppSecurity: TAppSecurity; AUserID: Integer): Boolean;
begin
  with TfmUserEditor.Create(Application) do
  try
    FAppSecurity := AppSecurity;
    FUserID := AUserID;
    Result := ShowModal = mrOk
  finally
    Free
  end;
end;

{ TfmUserEditor }

procedure TfmUserEditor.FormShow(Sender: TObject);
begin
  LoadData(FUserID)
end;

procedure TfmUserEditor.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if ModalResult = mrOk then CanClose := SaveData(FUserID);
end;

procedure TfmUserEditor.cbRoleKeyDown(Sender: TObject; var Key: Word; Shift:
  TShiftState);
begin
  if Key = VK_DELETE then
  begin
    cbRole.ItemIndex := -1;
    Key := 0;
  end;
end;

procedure TfmUserEditor.LoadData(AUserID: Integer);
var
  AUserName, ALogin, APassword: String;
  ARoleID: Integer;
begin
  LoadRoles;
  FAppSecurity.DataProvider.GetUserInfo(AUserID, AUserName, ALogin, APassword, ARoleID);
  edUserName.Text := AUserName;
  edLogin.Text := ALogin;
  edPassword.Text := APassword;
  cbRole.ItemIndex := cbRole.Items.IndexOfObject(TObject(ARoleID));
end;

procedure TfmUserEditor.LoadRoles;
begin
  with FAppSecurity.DataProvider.GetRolesTable do
  try
    First;
    with FAppSecurity.DataProvider.DBStructure do
    while Not Eof do
    begin
      if FieldByName(RoleTableObjectTypeField).AsInteger <> 0 then
        cbRole.Items.AddObject(FieldByName(RoleTableNameField).AsString,
          TObject(FieldByName(RoleTableRoleIDField).AsInteger));
      Next;
    end
  finally
    Free
  end;
end;

function TfmUserEditor.SaveData(AUserID: Integer): Boolean;
var
  AUserName, ALogin, APassword: String;
  ARoleID: Integer;
begin
  if (FAppSecurity.MinPasswordLength > 0) and (Length(edPassword.Text) < FAppSecurity.MinPasswordLength) then
  begin
    Application.MessageBox(PChar(Format('      %d .', [FAppSecurity.MinPasswordLength])),
    ' ', MB_ICONWARNING or MB_OK);
    Result := False;
  end
  else
  if Not IsValidLogin then
  begin
    Application.MessageBox('   .',
    ' ', MB_ICONWARNING or MB_OK);
    Result := False;
  end
  else
  begin
    AUserName := edUserName.Text;
    ALogin := edLogin.Text;
    APassword := edPassword.Text;
    if cbRole.ItemIndex >= 0 then
      ARoleID := Integer(cbRole.Items.Objects[cbRole.ItemIndex])
    else
      ARoleID := 0;
    with FAppSecurity do
    begin
      DataProvider.SetUserInfo(AUserID, AUserName, ALogin, APassword, ARoleID);
      if LogOptions.RegisterUserAccountChange then
        AddLogRecord(lmUserAccountChange, LogOptions.UserAccountChangeMessage + AUserName);
    end;
    Result := True;
  end;
end;

function TfmUserEditor.IsValidLogin: Boolean;
var
  I: Integer;
  Users: TStrings;
begin
  Users := TStringList.Create;
  try
    FAppSecurity.DataProvider.GetUsers(Users, True);
    I := Users.IndexOf(edLogin.Text);
    Result := (I < 0) or (FUserID = Integer(Users.Objects[I]));
  finally
    Users.Free;
  end;
end;

end.
