unit asfADO;

interface

uses SysUtils, Classes, DB, ADODB, asfData;

type
  TADOAppSecurityDataProvider = class(TAppSecurityDataProvider)
  private
    FConnection: TADOConnection;
    procedure SetConnection(Value: TADOConnection);
    procedure CheckConnection;
  protected
    procedure CreateTable(const ATableName: String; AFields: TFieldDefs; AIndexes:
      TIndexDefs); override;
    procedure ExecuteQuery(Query: String); override;
    function OpenQuery(Query: String): TDataSet; override;
    function OpenTable(const ATableName: String): TDataSet;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  published
    property Connection: TADOConnection read FConnection write SetConnection;
  end;

procedure CreateADOTable(AConnection: TADOConnection; const ATableName: String;
  AFields: TFieldDefs; AIndexes: TIndexDefs);

implementation

uses Variants, TypInfo;

resourcestring
  SCreateIndexSQL =
    'CREATE INDEX [%s] ON [%s](%s);';
  SCreateTableSQL =
    'CREATE TABLE [%s] (%s);';
   SAlterTableSQL = 'ALTER TABLE [%s] ';

{$DEFINE DELPHI11}

function GetSQLFieldType(FieldType: TFieldType): string;
const
{$IFDEF DELPHI11}
  SQLFieldTypes: array [TFieldType] of String =
   ({ftUnknown}'', {ftString}'varchar', {ftSmallint}'Smallint', {ftInteger}'int', {ftWord}'int',
    {ftBoolean}'Smallint', {ftFloat}'Float', {ftCurrency}'Money', {ftBCD}'Money', {ftDate}'DateTime', {ftTime}'DateTime', {ftDateTime}'DateTime',
    {ftBytes}'Binary', {ftVarBytes}'VarBinary', {ftAutoInc}'int', {ftBlob}'VarBinary(MAX)', {ftMemo}'Text', {ftGraphic}'Image', {ftFmtMemo}'Text',
    {ftParadoxOle}'', {ftDBaseOle}'', {ftTypedBinary}'Binary', {ftCursor}'Cursor', {ftFixedChar}'Char', {ftWideString}'NVarChar',
    {ftLargeint}'BigInt', {ftADT}'', {ftArray}'', {ftReference}'', {ftDataSet}'', {ftOraBlob}'', {ftOraClob}'',
    {ftVariant}'SQL_variant', {ftInterface}'', {ftIDispatch}'', {ftGuid}'Uniqueidentifier', {ftTimeStamp}'TimeStamp', {ftFMTBcd}'Decimal',
    '', '', '', '');
{$ELSE}
{$IFDEF DELPHI14}
  SQLFieldTypes: array [TFieldType] of String =
   ({ftUnknown}'', {ftString}'varchar', {ftSmallint}'Smallint', {ftInteger}'int', {ftWord}'int',
    {ftBoolean}'Smallint', {ftFloat}'Float', {ftCurrency}'Money', {ftBCD}'Money', {ftDate}'DateTime', {ftTime}'DateTime', {ftDateTime}'DateTime',
    {ftBytes}'Binary', {ftVarBytes}'VarBinary', {ftAutoInc}'int', {ftBlob}'VarBinary(MAX)', {ftMemo}'Text', {ftGraphic}'Image', {ftFmtMemo}'Text',
    {ftParadoxOle}'', {ftDBaseOle}'', {ftTypedBinary}'Binary', {ftCursor}'Cursor', {ftFixedChar}'Char', {ftWideString}'NVarChar',
    {ftLargeint}'BigInt', {ftADT}'', {ftArray}'', {ftReference}'', {ftDataSet}'', {ftOraBlob}'', {ftOraClob}'',
    {ftVariant}'SQL_variant', {ftInterface}'', {ftIDispatch}'', {ftGuid}'Uniqueidentifier', {ftTimeStamp}'TimeStamp', {ftFMTBcd}'Decimal',
    {ftFixedWideChar}'', {ftWideMemo}'', {ftOraTimeStamp}'', {ftOraInterval}'',
    {ftLongWord}'int', {ftShortint}'Smallint', {ftByte}'Smallint', {ftExtended}'Float', {ftConnection}'', {ftParams}'', {ftStream}'',
    {ftTimeStampOffset}'', {ftObject}'', {ftSingle}'Float'); //49..51
{$ELSE}
  SQLFieldTypes: array [TFieldType] of String =
   ({ftUnknown}'', {ftString}'varchar', {ftSmallint}'Smallint',
    {ftInteger}'int', {ftWord}'int', {ftBoolean}'Smallint',
    {ftFloat}'Float', {ftCurrency}'Money', {ftBCD}'Money',
    {ftDate}'DateTime', {ftTime}'DateTime', {ftDateTime}'DateTime',
    {ftBytes}'Binary', {ftVarBytes}'VarBinary', {ftAutoInc}'int',
    {ftBlob}'VarBinary(MAX)', {ftMemo}'Text',
    {ftGraphic}'Image', {ftFmtMemo}'Text',
    {ftParadoxOle}'', {ftDBaseOle}'',
    {ftTypedBinary}'Binary',
    {ftCursor}'Cursor', {ftFixedChar}'Char', {ftWideString}'NVarChar',
    {ftLargeint}'BigInt', {ftADT}'', {ftArray}'',
    {ftReference}'', {ftDataSet}'', {ftOraBlob}'',
    {ftOraClob}'', {ftVariant}'SQL_variant', {ftInterface}'',
    {ftIDispatch}'', {ftGuid}'Uniqueidentifier', {ftTimeStamp}'TimeStamp',
    {ftFMTBcd}'Decimal');
{$ENDIF}
{$ENDIF}
begin
  Result := SQLFieldTypes[FieldType];
  if Result = '' then
    raise Exception.Create(' :' +
      GetEnumName(TypeInfo(TFieldType), Integer(FieldType)));
end;

{{
     SQL-.
}
function GetSQLFieldDef(AFieldName: String; AFieldType: TDataType; AFieldSize:
  Integer; ARequired: Boolean): String;
begin
  Result := Format('[%s] %s', [AFieldName, GetSQLFieldType(AFieldType)]);

  if AFieldType in [ftString] then
  begin
    if AFieldSize <= 0 then
      raise Exception.CreateFmt('  (%d)   %s', [AFieldSize, AFieldName]);
    Result := Result + Format('(%d)', [AFieldSize]);
  end else
  if AFieldType = ftAutoInc then
    Result := Result + ' IDENTITY(1,1)';

  if ARequired then Result := Result + ' NOT NULL';
end;

function ExtractSubStr(var S: String; Delimiter: String): String;
var
  P: Integer;
begin
  P := Pos(Delimiter, S);
  if P > 0 then begin
    Result := Copy(S, 1, P-1);
    S := Copy(S, P+1, Length(S));
  end
  else
  begin
    Result := S;
    S := '';
  end;
end;

function TableExists(AConnection: TADOConnection; ATableName: String): Boolean;
var
  Filter: OleVariant;
  ADataSet: TADODataSet;
begin
  Filter := VarArrayCreate([0, 3], varVariant);
  Filter[2] := ATableName;
  ADataSet := TADODataSet.Create(nil);
  try
    AConnection.OpenSchema(siTables, Filter, EmptyParam, ADataSet);
    Result := ADataSet.RecordCount > 0;
  finally
    ADataSet.Free
  end;
end;

procedure ExecuteSQLQuery(AConnection: TADOConnection; SQLText: String);
begin
  with TADOQuery.Create(nil) do
  try
    Connection := AConnection;
    SQL.Text := SQLText;
    ExecSQL;
  finally
    Free
  end;
end;

procedure ExecAlterTableQuery(AConnection: TADOConnection; TableName: String;
  NewFields, ChangedFields: TStrings);
var
  I: Integer;
  Query: String;
begin
  for I := 0 to NewFields.Count - 1 do
  begin
    Query := Format(SAlterTableSQL, [TableName]) + 'ADD ' + NewFields[I];
    ExecuteSQLQuery(AConnection, Query);
  end;
  for I := 0 to ChangedFields.Count - 1 do
  begin
    Query := Format(SAlterTableSQL, [TableName]) + 'ALTER COLUMN ' + ChangedFields[I];
    ExecuteSQLQuery(AConnection, Query);
  end;
end;

procedure RestructureTableInternal(AConnection: TADOConnection; ATableName:
  String; AFieldsDefs: TFieldDefs; AnIndexDefs: TIndexDefs);
var
  I: Integer;
  NewFields, ChangedFields: TStringList;
  FieldDef: TFieldDef;
  ATable: TADOTable;
begin
  ATable := TADOTable.Create(nil);
  try
    ATable.Connection := AConnection;
    ATable.TableName := ATableName;
    ATable.FieldDefs.Update;

    NewFields := TStringList.Create;
    ChangedFields := TStringList.Create;
    try
      //      
      for I := 0 to AFieldsDefs.Count - 1 do
      begin
        FieldDef := nil;
        try
          FieldDef := ATable.FieldDefs.Find(AFieldsDefs[I].Name);
        except
          //    ,   EDatabaseError
          on EDatabaseError: Exception do ;
        end;

        if Not Assigned(FieldDef) then
          with AFieldsDefs[I] do
          NewFields.Add(GetSQLFieldDef(Name, DataType, Size, Required))
        else
        if (GetSQLFieldType(FieldDef.DataType) <> GetSQLFieldType(AFieldsDefs[I].DataType)) or
          (FieldDef.Size <> AFieldsDefs[I].Size)
        then
          with AFieldsDefs[I] do
          ChangedFields.Add(GetSQLFieldDef(Name, DataType, Size, Required))
      end;
      ExecAlterTableQuery(AConnection, ATableName, NewFields, ChangedFields);
    finally
      ChangedFields.Free;
      NewFields.Free;
    end;

  finally
    ATable.Free
  end;
end;

function RestructureTable(AConnection: TADOConnection; ATableName: String;
  AFieldsDefs: TFieldDefs; AnIndexDefs: TIndexDefs): Boolean;
begin
  if Not TableExists(AConnection, ATableName) then
    //    -   
    Result := False
  else
  begin
    //   -  
    RestructureTableInternal(AConnection, ATableName, AFieldsDefs, AnIndexDefs);
    //  
    Result := True;
  end;
end;

procedure GetCreateTableQuery(ATableName: String;
    AFieldsDefs: TFieldDefs; AnIndexDefs: TIndexDefs; AQuery: TStrings);

  function GetFieldListSQL: String;
  var
    I: Integer;
  begin
    Result := '';
    for I := 0 to AFieldsDefs.Count - 1 do
    begin
      with AFieldsDefs[I] do
        Result := Result + GetSQLFieldDef(Name, DataType, Size, Required);
      if I < AFieldsDefs.Count - 1 then
        Result := Result + ', ';
    end;
  end;

  {{
     .
  }
  procedure AddIndexQuery;
  var
    FieldList, IndexName, S: String;
    I: Integer;
    FieldName: String;
  begin
    for I := 0 to AnIndexDefs.Count - 1 do
    begin
      IndexName := AnIndexDefs[I].Name;

      //  ;  ,   []
      S := AnIndexDefs[I].Fields;
      FieldList := '';
      while S <> '' do
      begin
        FieldName := ExtractSubStr(S, ';');
        FieldList := FieldList + ',[' + FieldName + ']';
      end;
      Delete(FieldList, 1, 1);

      S := Format(SCreateIndexSQL, [IndexName, ATableName, FieldList]);
      AQuery.Add(S);
    end;
  end;

begin
  AQuery.Add(Format(SCreateTableSQL, [ATableName, GetFieldListSQL]));
  AddIndexQuery;
end;

procedure CreateADOTable(AConnection: TADOConnection; const ATableName: String;
  AFields: TFieldDefs; AIndexes: TIndexDefs);
var
  I: Integer;
  Query: TStringList;
begin
  if Not RestructureTable(AConnection, ATableName, AFields, AIndexes) then
  begin
    Query := TStringList.Create;
    try
      GetCreateTableQuery(ATableName, AFields, AIndexes, Query);
      for I := 0 to Query.Count - 1 do
        ExecuteSQLQuery(AConnection, Query[I]);
    finally
      Query.Free
    end;
  end
end;

{ TADOAppSecurityDataProvider }

procedure TADOAppSecurityDataProvider.SetConnection(Value: TADOConnection);
begin
  if FConnection <> Value then
  begin
    FConnection := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TADOAppSecurityDataProvider.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = Connection) and (Operation = opRemove)
    then Connection := nil;
end;

procedure TADOAppSecurityDataProvider.CheckConnection;
begin
  if Not Assigned(Connection) then
    raise Exception.Create('   Connection  ' + Name);
end;

procedure TADOAppSecurityDataProvider.CreateTable(const ATableName: String;
  AFields: TFieldDefs; AIndexes: TIndexDefs);
begin
  CheckConnection;
  CreateADOTable(Connection, ATableName, AFields, AIndexes);
end;

function TADOAppSecurityDataProvider.OpenTable(const ATableName: String):
  TDataSet;
var
  ATable: TADOTable;
begin
  CheckConnection;
  ATable := TADOTable.Create(Self);
  try
    ATable.Connection := Connection;
    ATable.TableName := ATableName;
    ATable.Open;
    Result := ATable;
  except
    ATable.Free; raise
  end;
end;

procedure TADOAppSecurityDataProvider.ExecuteQuery(Query: String);
var
  AQuery: TADOQuery;
begin
  CheckConnection;
  AQuery := TADOQuery.Create(Self);
  try
    AQuery.Connection := Connection;
    AQuery.SQL.Text := Query;
    AQuery.ExecSQL;
  finally
    AQuery.Free;
  end;
end;

function TADOAppSecurityDataProvider.OpenQuery(Query: String): TDataSet;
var
  AQuery: TADOQuery;
begin
  CheckConnection;
  AQuery := TADOQuery.Create(Self);
  try
    AQuery.Connection := Connection;
    AQuery.SQL.Text := Query;
    AQuery.Open;
    Result := AQuery;
  except
    AQuery.Free; raise
  end;
end;

end.
