unit asfActions;

interface

uses
  SysUtils, Classes, ActnList;

type
  TAppSecurityAction = class(TAction)
  private
    FDisabled: Boolean;
  public
    procedure UpdateTarget(Target: TObject); override;
    function HandlesTarget(Target: TObject): Boolean; override;
  published
    property Disabled: Boolean read FDisabled write FDisabled stored False default False;
  end;

  TAppSecurityFormAccessItem = class(TComponent)
  private
    FEnabled: Boolean;
    FOnAccessStateChanged: TNotifyEvent;
    procedure SetEnabled(const Value: Boolean);
  protected
    procedure AccessStateChanged;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Enabled: Boolean read FEnabled write SetEnabled stored False default True;
    property OnAccessStateChanged: TNotifyEvent read FOnAccessStateChanged write
      FOnAccessStateChanged;

    property Tag;
  end;

implementation

{ TAppSecurityAction }

function TAppSecurityAction.HandlesTarget(Target: TObject): Boolean;
begin
  Result := True
end;

procedure TAppSecurityAction.UpdateTarget(Target: TObject);
begin
  if Disabled then
    Enabled := False
  else
    inherited
end;

{ TAppSecurityFormAccessItem }

constructor TAppSecurityFormAccessItem.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FEnabled := True;
end;

procedure TAppSecurityFormAccessItem.SetEnabled(const Value: Boolean);
begin
  FEnabled := Value;
  AccessStateChanged;
end;

procedure TAppSecurityFormAccessItem.AccessStateChanged;
begin
  if Assigned(FOnAccessStateChanged) then FOnAccessStateChanged(Self);
end;

end.
