unit asfData;

interface

uses SysUtils, Classes, DB;

type
  TAppSecurityDBStructure = class(TPersistent)
  private
    FUserTable: String;
    FUserTableUserIDField: String;
    FUserTableLoginField: String;
    FUserTablePaswordField: String;
    FUserTableCheckSumField: String;
    FRightsTable: String;
    FRightsTableCheckSumField: String;
    FRightsTableItemIDField: String;
    FRightsTableObjectIDField: String;
    FRightsTableAccessStateField: String;
    FUserTableFullNameField: String;
    FUserTableParentIDField: String;
    FUserTableObjectTypeField: String;
    FRoleTable: String;
    FRoleTableNameField: String;
    FRoleTableObjectTypeField: String;
    FRoleTableParentIDField: String;
    FRoleTableRoleIDField: String;
    FUserTableRoleIDField: String;
    FRightsTableObjectTypeField: String;
    FLogTable: String;
    FLogTableMessageField: String;
    FLogTableMessageTypeField: String;
    FLogTableDateField: String;
    FLogTableUserIDField: String;
    FLogTableUserNameField: String;
    FLogTableIDField: String;
    FRightsTableIDField: String;
    FLogTableComputerNameField: String;
  public
    constructor Create;
  published
    property UserTable: String read FUserTable write FUserTable;
    property UserTableUserIDField: String read FUserTableUserIDField write
      FUserTableUserIDField;
    property UserTableParentIDField: String read FUserTableParentIDField write
      FUserTableParentIDField;
    property UserTableFullNameField: String read FUserTableFullNameField write
      FUserTableFullNameField;
    property UserTableLoginField: String read FUserTableLoginField write
      FUserTableLoginField;
    property UserTablePaswordField: String read FUserTablePaswordField write
      FUserTablePaswordField;
    property UserTableObjectTypeField: String read FUserTableObjectTypeField write
      FUserTableObjectTypeField;
    property UserTableRoleIDField: String read FUserTableRoleIDField write
      FUserTableRoleIDField;
    property UserTableCheckSumField: String read FUserTableCheckSumField write
      FUserTableCheckSumField;

    property RoleTable: String read FRoleTable write FRoleTable;
    property RoleTableNameField: String read FRoleTableNameField write
      FRoleTableNameField;
    property RoleTableObjectTypeField: String read FRoleTableObjectTypeField write
      FRoleTableObjectTypeField;
    property RoleTableParentIDField: String read FRoleTableParentIDField write
      FRoleTableParentIDField;
    property RoleTableRoleIDField: String read FRoleTableRoleIDField write
      FRoleTableRoleIDField;
    property RightsTable: String read FRightsTable write FRightsTable;
    property RightsTableIDField: String read FRightsTableIDField write
      FRightsTableIDField;
    property RightsTableItemIDField: String read FRightsTableItemIDField write
      FRightsTableItemIDField;
    property RightsTableObjectIDField: String read FRightsTableObjectIDField write
      FRightsTableObjectIDField;
    property RightsTableObjectTypeField: String read FRightsTableObjectTypeField
      write FRightsTableObjectTypeField;
    property RightsTableAccessStateField: String read FRightsTableAccessStateField
      write FRightsTableAccessStateField;
    property RightsTableCheckSumField: String read FRightsTableCheckSumField write
      FRightsTableCheckSumField;

    property LogTable: String read FLogTable write FLogTable;
    property LogTableIDField: String read FLogTableIDField write FLogTableIDField;
    property LogTableDateField: String read FLogTableDateField write
      FLogTableDateField;
    property LogTableMessageTypeField: String read FLogTableMessageTypeField write
      FLogTableMessageTypeField;
    property LogTableMessageField: String read FLogTableMessageField write
      FLogTableMessageField;
    property LogTableUserIDField: String read FLogTableUserIDField write
      FLogTableUserIDField;
    property LogTableUserNameField: String read FLogTableUserNameField write
      FLogTableUserNameField;
    property LogTableComputerNameField: String read FLogTableComputerNameField
      write FLogTableComputerNameField;
  end;

  TCreateTableEvent = procedure(Sender: TObject; const ATableName: String;
    AFields: TFieldDefs; AIndexes: TIndexDefs) of object;
  TOpenTableEvent = procedure(Sender: TObject; const ATableName: String; var DataSet:
    TDataSet) of object;
  TOpenQueryEvent = procedure(Sender: TObject; const Query: String; var DataSet:
    TDataSet) of object;
  TExecuteQueryEvent = procedure(Sender: TObject; const Query: String) of object;
  TAddLogDataEvent = procedure(Sender: TObject; ALogRecordID: Integer; AUserData:
    TObject) of object;
  TGetLogDataEvent = procedure(Sender: TObject; ALogRecordID: Integer; var
    AUserData: TObject) of object;

  TAppSecurityDataProvider = class(TComponent)
  private
    FDBStructure: TAppSecurityDBStructure;
    FOnOpenQuery: TOpenQueryEvent;
    FOnCreateTable: TCreateTableEvent;
    FOnOpenTable: TOpenTableEvent;
    FOnExecuteQuery: TExecuteQueryEvent;
    FOnAddLogData: TAddLogDataEvent;
    FDateTimeFormat: String;
    FOnGetLogData: TGetLogDataEvent;
    procedure SetDBStructure(const Value: TAppSecurityDBStructure);
    procedure NotImplementedOperation(OperationName: String);
    procedure MakeRecordCRC(const ATableName: String; ARecordIDFieldName: String;
      ARecordID: Integer; ACRCFieldName: String; AIgnoreFields: String = '');
  protected
    function OpenQuery(Query: String): TDataSet; dynamic;
    procedure CreateTable(const ATableName: String; AFields: TFieldDefs; AIndexes:
      TIndexDefs); dynamic;
    function IsTableExists(const ATableName: String): Boolean; dynamic;
    function OpenTable(const ATableName: String): TDataSet;
    procedure ExecuteQuery(Query: String); dynamic;
    procedure AddLogData(ALogRecordID: Integer; AUserData: TObject); dynamic;
    function GetMaxValue(const ATableName: String; AFileName: String): Integer;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GenerateDatabase: Boolean;
    function GetUserPassword(UserID: Integer): String;
    procedure GetUsers(Users: TStrings; GetLogins: Boolean);
    function GetUsersTable: TDataSet;
    function GetRolesTable: TDataSet;
    procedure GetUserInfo(AUserID: Integer; var AUserName, ALogin, APassword:
      String; var ARoleID: Integer);
    procedure SetUserInfo(AUserID: Integer; AUserName, ALogin, APassword: String;
      ARoleID: Integer);
    procedure ClearObjectRights(ObjectID, ObjectType: Integer);
    procedure SetObjectRights(ObjectID, ObjectType, ModelItemID: Integer;
      AccessKind: String);
    function GetObjectRights(ObjectID, ObjectType, ModelItemID: Integer; var
      AccessKind: String): Boolean;
    procedure AddLogRecord(ADateTime: TDateTime; AUserID: Integer; AUserName,
      AComputerName: String; AMessageType: Integer; AMessage: String; AUserData:
      TObject);
    function AddUser(AUserName: String): Integer;
    function GetLogRecord(ALogRecordID: Integer): TDataSet; dynamic;
    function GetLastLogRecordID: Integer; overload;
    function GetLogRecords(AStartDate, AEndDate: TDateTime): TDataSet;
    procedure DeleteLogRecords(AStartDate, AEndDate: TDateTime);
    function GetLogData(ALogRecordID: Integer): TObject;
    function GetLastLogRecordID(ALogRecordType: Integer; ALogRecordMessage:
      String): Integer; overload;
  published
    property DBStructure: TAppSecurityDBStructure read FDBStructure write
      SetDBStructure;
    property DateTimeFormat: String read FDateTimeFormat write FDateTimeFormat;
    property OnOpenQuery: TOpenQueryEvent read FOnOpenQuery write FOnOpenQuery;
    property OnExecuteQuery: TExecuteQueryEvent read FOnExecuteQuery write
      FOnExecuteQuery;
    property OnCreateTable: TCreateTableEvent read FOnCreateTable write
      FOnCreateTable;
    property OnOpenTable: TOpenTableEvent read FOnOpenTable write FOnOpenTable;
    property OnAddLogData: TAddLogDataEvent read FOnAddLogData write FOnAddLogData;
    property OnGetLogData: TGetLogDataEvent read FOnGetLogData write FOnGetLogData;
  end;

  EOperationNotImplemented = class (Exception)
  end;

const
  {   }
  GroupObjectType = 0;
  UserObjectType = 1;
  RoleObjectType = 2;

implementation

uses asfUtils;

const
  MaxNameFieldLen = 100;
  EncodedTextLen = 200;

type
  TTableStructure = class
  private
    FIndexDefs: TIndexDefs;
    FDataSet: TDataSet;
    function GetFieldDefs: TFieldDefs;
  public
    constructor Create;
    destructor Destroy; override;
    procedure AddField(FieldName: String; FieldType: TFieldType; FieldSize: Integer
      = 0; Required: Boolean = False);
    procedure AddIndex(IndexName, IndexFields: String; IndexOptions: TIndexOptions);
    procedure Clear;
    property FieldDefs: TFieldDefs read GetFieldDefs;
    property IndexDefs: TIndexDefs read FIndexDefs;
  end;

{ TTableStructure }

constructor TTableStructure.Create;
begin
  inherited Create;
  FDataSet := TDataSet.Create(nil);
  FIndexDefs := TIndexDefs.Create(FDataSet);
end;

destructor TTableStructure.Destroy;
begin
  FreeAndNil(FIndexDefs);
  FreeAndNil(FDataSet);
  inherited Destroy;
end;

function TTableStructure.GetFieldDefs: TFieldDefs;
begin
  Result := FDataSet.FieldDefs;
end;

procedure TTableStructure.AddField(FieldName: String; FieldType: TFieldType;
  FieldSize: Integer = 0; Required: Boolean = False);
begin
  with FieldDefs do
    if IndexOf(FieldName) < 0 then
      Add(FieldName, FieldType, FieldSize, Required);
end;

procedure TTableStructure.AddIndex(IndexName, IndexFields: String; IndexOptions:
  TIndexOptions);
begin
  IndexDefs.Add(IndexName, IndexFields, IndexOptions);
end;

procedure TTableStructure.Clear;
begin
  FieldDefs.Clear;
  IndexDefs.Clear;
end;

{ TAppSecurityDBStructure }

constructor TAppSecurityDBStructure.Create;
begin
  inherited Create;

  FUserTable := 'ASFUsers';
  FUserTableUserIDField := 'UserID';
  FUserTableParentIDField := 'ParentID';
  FUserTableFullNameField := 'FullName';
  FUserTableLoginField := 'Login';
  FUserTablePaswordField := 'Pasword';
  FUserTableObjectTypeField := 'ObjectType';
  FUserTableRoleIDField := 'RoleID';
  FUserTableCheckSumField := 'CRC';

  FRoleTable := 'ASFRoles';
  FRoleTableRoleIDField := 'RoleID';
  FRoleTableParentIDField := 'ParentID';
  FRoleTableNameField := 'Name';
  FRoleTableObjectTypeField := 'ObjectType';

  FRightsTable := 'ASFRights';
  FRightsTableIDField := 'RecordID';
  FRightsTableItemIDField := 'ItemID';
  FRightsTableObjectIDField := 'ObjectID';
  FRightsTableObjectTypeField := 'ObjectType';
  FRightsTableAccessStateField := 'Access';
  FRightsTableCheckSumField := 'CRC';

  LogTable := 'ASFLog';
  LogTableIDField := 'RecordID';
  LogTableDateField := 'DateTime';
  LogTableMessageTypeField := 'MessageType';
  LogTableMessageField := 'Message';
  LogTableUserIDField := 'UserID';
  LogTableUserNameField := 'UserName';
  LogTableComputerNameField := 'Computer';
end;

{ TAppSecurityDataProvider }

constructor TAppSecurityDataProvider.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDBStructure := TAppSecurityDBStructure.Create;
  FDateTimeFormat := 'dd"/"mm"/"yyyy hh":"nn":"ss';
end;

destructor TAppSecurityDataProvider.Destroy;
begin
  FreeAndNil(FDBStructure);
  inherited Destroy;
end;

procedure TAppSecurityDataProvider.SetDBStructure(const Value:
  TAppSecurityDBStructure);
begin
end;

{{
      (    ).
  True,   UserTable     
 (      ),  
   False.
}
function TAppSecurityDataProvider.GenerateDatabase: Boolean;
var
  TableStructure: TTableStructure;
begin
  TableStructure := TTableStructure.Create;
  try
    with DBStructure, TableStructure do
    begin
      {   ------------------------------------------------ }

      if Not IsTableExists(UserTable) then
      begin
        Clear;
        {   }
        AddField(UserTableUserIDField, ftAutoInc);
        AddField(UserTableParentIDField, ftInteger);
        AddField(UserTableFullNameField, ftString, MaxNameFieldLen);
        AddField(UserTableLoginField, ftString, EncodedTextLen);
        AddField(UserTablePaswordField, ftString, EncodedTextLen);
        AddField(UserTableObjectTypeField, ftSmallint);
        AddField(UserTableRoleIDField, ftInteger);
        AddField(UserTableCheckSumField, ftInteger);

        {   }
        AddIndex('ByUserID', UserTableUserIDField, [ixPrimary, ixUnique]);

        CreateTable(UserTable, FieldDefs, IndexDefs);
        Result := True;
      end else
        Result := False;

      {   -------------------------------------------------------- }

      if Not IsTableExists(RoleTable) then
      begin
        Clear;
        {   }
        AddField(RoleTableRoleIDField, ftAutoInc);
        AddField(RoleTableParentIDField, ftInteger);
        AddField(RoleTableNameField, ftString, MaxNameFieldLen);
        AddField(RoleTableObjectTypeField, ftSmallint);

        {   }
        AddIndex('ByRoleID', RoleTableRoleIDField, [ixPrimary, ixUnique]);

        CreateTable(RoleTable, FieldDefs, IndexDefs);
      end;

      {   --------------------------------------------------------- }

      if Not IsTableExists(RightsTable) then
      begin
        Clear;
        {   }
        AddField(RightsTableIDField, ftAutoInc);
        AddField(RightsTableItemIDField, ftInteger);
        AddField(RightsTableObjectIDField, ftInteger);
        AddField(RightsTableObjectTypeField, ftInteger);
        AddField(RightsTableAccessStateField, ftString, 30);
        AddField(RightsTableCheckSumField, ftInteger);

        {   }
        AddIndex('ByID', RightsTableIDField, [ixPrimary, ixUnique]);

        CreateTable(RightsTable, FieldDefs, IndexDefs);
      end;

      {   --------------------------------------------- }

      if Not IsTableExists(LogTable) then
      begin
        Clear;
        {   }
        AddField(LogTableIDField, ftAutoInc);
        AddField(LogTableUserIDField, ftInteger);
        AddField(LogTableUserNameField, ftString, MaxNameFieldLen);
        AddField(LogTableDateField, ftDateTime);
        AddField(LogTableComputerNameField, ftString, MaxNameFieldLen);
        AddField(LogTableMessageTypeField, ftInteger);
        AddField(LogTableMessageField, ftString, 255);

        {   }
        AddIndex('ByID', LogTableIDField, [ixPrimary, ixUnique]);

        CreateTable(LogTable, FieldDefs, IndexDefs);
      end;

    end;
  finally
    TableStructure.Free;
  end;
end;

function TAppSecurityDataProvider.GetMaxValue(const ATableName: String;
  AFileName: String): Integer;
var
  Query: String;
begin
  Query := Format('SELECT Max(%s) FROM %s', [AFileName, ATableName]);
  with OpenQuery(Query) do
  try
    if Not IsEmpty then
      Result := Fields[0].AsInteger
    else
      Result := 0
  finally
    Free
  end;
end;

{***************************************************************************** }
{                                                       }
{ **************************************************************************** }

function TAppSecurityDataProvider.OpenQuery(Query: String): TDataSet;
begin
  if Assigned(FOnOpenQuery) then
    FOnOpenQuery(Self, Query, Result)
  else
    NotImplementedOperation('OpenQuery');
end;

procedure TAppSecurityDataProvider.ExecuteQuery(Query: String);
begin
  if Assigned(FOnExecuteQuery) then
    FOnExecuteQuery(Self, Query)
  else
    NotImplementedOperation('ExecuteQuery');
end;

function TAppSecurityDataProvider.OpenTable(const ATableName: String): TDataSet;
begin
  if Assigned(FOnOpenTable) then
    FOnOpenTable(Self, ATableName, Result)
  else
    NotImplementedOperation('OpenTable');
end;

procedure TAppSecurityDataProvider.CreateTable(const ATableName: String;
  AFields: TFieldDefs; AIndexes: TIndexDefs);
begin
  if Assigned(FOnCreateTable) then
    FOnCreateTable(Self, ATableName, AFields, AIndexes)
  else
    NotImplementedOperation('CreateTable');
end;

{{
       .   
     .    , 
  .
}
function TAppSecurityDataProvider.IsTableExists(
  const ATableName: String): Boolean;
var
  DataSet: TDataSet;
  Query: String;
begin
  with DBStructure do
  begin
    Query := Format('SELECT TOP 1 * FROM %s', [ATableName]);
    try
      DataSet := OpenQuery(Query);
    except
      Result := False; Exit;
    end;
    DataSet.Free;
    Result := True;
  end;
end;

procedure TAppSecurityDataProvider.AddLogData(ALogRecordID: Integer; AUserData:
  TObject);
begin
  if Assigned(FOnAddLogData) then
    FOnAddLogData(Self, ALogRecordID, AUserData)
  else
    NotImplementedOperation('AddLogData');
end;

function TAppSecurityDataProvider.GetLogData(ALogRecordID: Integer): TObject;
begin
  if Assigned(FOnGetLogData) then
    FOnGetLogData(Self, ALogRecordID, Result)
  else
    NotImplementedOperation('GetLogData');
end;

function TAppSecurityDataProvider.GetLogRecord(ALogRecordID: Integer): TDataSet;
var
  Query: String;
begin
  with DBStructure do
  begin
    Query := Format('SELECT * FROM %s WHERE %s = %d', [LogTable, LogTableIDField, ALogRecordID]);
    Result := OpenQuery(Query);
  end;
end;

function TAppSecurityDataProvider.GetLogRecords(AStartDate, AEndDate:
  TDateTime): TDataSet;
var
  Query: String;
  SelectedFields: String;
begin
  with DBStructure do
  begin
    SelectedFields := LogTableIDField + ', ' + LogTableMessageTypeField + ', ' + LogTableDateField  + ', ' +
      LogTableUserNameField + ', ' + LogTableComputerNameField + ', ' + LogTableMessageField;
    Query := Format('SELECT %s FROM %s WHERE %s >= ''%s'' AND %s <= ''%s''', [SelectedFields, LogTable,
      LogTableDateField, FormatDateTime(DateTimeFormat, AStartDate), LogTableDateField, FormatDateTime(DateTimeFormat, AEndDate)]);
    Result := OpenQuery(Query);
  end;
end;

procedure TAppSecurityDataProvider.DeleteLogRecords(AStartDate, AEndDate:
  TDateTime);
var
  Query: String;
begin
  with DBStructure do
  begin
    Query := Format('DELETE FROM %s WHERE %s >= ''%s'' AND %s <= ''%s''', [LogTable,
      LogTableDateField, FormatDateTime(DateTimeFormat, AStartDate), LogTableDateField, FormatDateTime(DateTimeFormat, AEndDate)]);
    ExecuteQuery(Query);
  end;
end;

procedure TAppSecurityDataProvider.NotImplementedOperation(OperationName:
  String);
begin
  raise EOperationNotImplemented.CreateFmt(' %s  ', [OperationName]);
end;

{***************************************************************************** }
{                                                         }
{ **************************************************************************** }

function TAppSecurityDataProvider.GetUserPassword(UserID: Integer): String;
var
  DataSet: TDataSet;
  Query: String;
begin
  with DBStructure do
  begin
    Query := Format('SELECT * FROM %s WHERE %s = %d', [UserTable, UserTableUserIDField, UserID]);

    DataSet := OpenQuery(Query);
    try
      CheckRecordCRC(DataSet, UserTable, UserTableCheckSumField, UserTableParentIDField);
      Result := DecodeText(DataSet.FieldByName(UserTablePaswordField).AsString)
    finally
      DataSet.Free;
    end;
  end;
end;

procedure TAppSecurityDataProvider.GetUsers(Users: TStrings; GetLogins:
  Boolean);
var
  DataSet: TDataSet;
  Query: String;
begin
  with DBStructure do
  begin
    if GetLogins then
      Query := Format('SELECT %s, %s FROM %s WHERE %s = 1',
              [UserTableLoginField, UserTableUserIDField, UserTable, UserTableObjectTypeField])
    else
      Query := Format('SELECT %s, %s FROM %s WHERE %s = 1',
        [UserTableFullNameField, UserTableUserIDField, UserTable, UserTableObjectTypeField]);

    DataSet := OpenQuery(Query);
    try
      Users.Clear;
      while Not DataSet.Eof do
      begin
        if GetLogins then
          Users.AddObject(DecodeText(DataSet.Fields[0].AsString), TObject(DataSet.Fields[1].AsInteger))
        else
          Users.AddObject(DataSet.Fields[0].AsString, TObject(DataSet.Fields[1].AsInteger));
        DataSet.Next;
      end;

    finally
      DataSet.Free;
    end;
  end;
end;

function TAppSecurityDataProvider.GetUsersTable: TDataSet;
begin
  Result := OpenTable(DBStructure.UserTable);
end;

function TAppSecurityDataProvider.GetRolesTable: TDataSet;
begin
  Result := OpenTable(DBStructure.RoleTable);
end;

procedure TAppSecurityDataProvider.GetUserInfo(AUserID: Integer; var AUserName,
  ALogin, APassword: String; var ARoleID: Integer);
var
  DataSet: TDataSet;
  Query: String;
begin
  with DBStructure do
  begin
    Query := Format('SELECT * FROM %s WHERE %s = %d', [UserTable, UserTableUserIDField, AUserID]);

    DataSet := OpenQuery(Query);
    try
      CheckRecordCRC(DataSet, UserTable, UserTableCheckSumField, UserTableParentIDField);
      AUserName := DataSet.FieldByName(UserTableFullNameField).AsString;
      ARoleID := DataSet.FieldByName(UserTableRoleIDField).AsInteger;
      ALogin := DecodeText(DataSet.FieldByName(UserTableLoginField).AsString);
      APassword := DecodeText(DataSet.FieldByName(UserTablePaswordField).AsString)
    finally
      DataSet.Free;
    end;
  end;
end;

procedure TAppSecurityDataProvider.SetUserInfo(AUserID: Integer; AUserName,
  ALogin, APassword: String; ARoleID: Integer);
var
  Query: String;
begin
  with DBStructure do
  begin
    ALogin := EncodeText(ALogin, EncodedTextLen);
    APassword := EncodeText(APassword, EncodedTextLen);
    Query := Format('UPDATE %s SET %s = ''%s'', %s = ''%s'', %s = ''%s'' , %s = %d WHERE %s = %d',
      [UserTable, UserTableFullNameField, AUserName, UserTableLoginField, ALogin,
       UserTablePaswordField, APassword, UserTableRoleIDField, ARoleID, UserTableUserIDField, AUserID]);
    ExecuteQuery(Query);
    MakeRecordCRC(UserTable, UserTableUserIDField, AUserID, UserTableCheckSumField, UserTableParentIDField);
  end;
end;

procedure TAppSecurityDataProvider.MakeRecordCRC(const ATableName: String;
  ARecordIDFieldName: String; ARecordID: Integer; ACRCFieldName: String;
  AIgnoreFields: String = '');
var
  ACRC: Integer;
  DataSet: TDataSet;
  Query: String;
begin
  Query := Format('SELECT * FROM %s WHERE %s = %d', [ATableName, ARecordIDFieldName, ARecordID]);
  DataSet := OpenQuery(Query);
  try
    ACRC := GetRecordCRC(DataSet, CombineSubStr(ACRCFieldName, ';', AIgnoreFields));
  finally
    DataSet.Free;
  end;

  Query := Format('UPDATE %s SET %s = %d WHERE %s = %d',
    [ATableName, ACRCFieldName, ACRC, ARecordIDFieldName, ARecordID]);
  ExecuteQuery(Query);
end;

function TAppSecurityDataProvider.AddUser(AUserName: String): Integer;
var
  Query: String;
begin
  with DBStructure do
  begin
    Query := Format('INSERT INTO %s (%s, %s) VALUES (''%s'', %d)',
      [UserTable, UserTableFullNameField, UserTableObjectTypeField, AUserName, UserObjectType]);
    ExecuteQuery(Query);

    Result := GetMaxValue(UserTable, UserTableUserIDField);
  end;
end;

procedure TAppSecurityDataProvider.ClearObjectRights(ObjectID, ObjectType:
  Integer);
var
  Query: String;
begin
  with DBStructure do
    Query := Format('DELETE FROM %s WHERE %s = %d AND %s = %d',
      [RightsTable, RightsTableObjectIDField, ObjectID,
         RightsTableObjectTypeField, ObjectType]);
  ExecuteQuery(Query);
end;

procedure TAppSecurityDataProvider.SetObjectRights(ObjectID, ObjectType,
  ModelItemID: Integer; AccessKind: String);
var
  ARecordID: Integer;
  Query: String;
begin
  with DBStructure do
  begin
    Query := Format('INSERT INTO %s (%s, %s, %s, %s) VALUES (%d, %d, %d, ''%s'')',
      [RightsTable,
       RightsTableObjectIDField, RightsTableObjectTypeField, RightsTableItemIDField, RightsTableAccessStateField,
        ObjectID, ObjectType, ModelItemID, AccessKind]);
    ExecuteQuery(Query);

    ARecordID := GetMaxValue(RightsTable, RightsTableIDField);
    MakeRecordCRC(RightsTable, RightsTableIDField, ARecordID, RightsTableCheckSumField);
  end;
end;

{{
        
  .
}
function TAppSecurityDataProvider.GetObjectRights(ObjectID, ObjectType,
  ModelItemID: Integer; var AccessKind: String): Boolean;
var
  DataSet: TDataSet;
  Query: String;
begin
  with DBStructure do
  begin
    Query := Format('SELECT * FROM %s WHERE %s = %d AND %s = %d AND %s = %d',
      [RightsTable, RightsTableObjectIDField, ObjectID,
       RightsTableObjectTypeField, ObjectType,
       RightsTableItemIDField, ModelItemID]);

    DataSet := OpenQuery(Query);
    try
      Result := Not DataSet.IsEmpty;
      if Result then
      begin
        CheckRecordCRC(DataSet, RightsTable, RightsTableCheckSumField);
        AccessKind := DataSet.FieldByName(RightsTableAccessStateField).AsString;
      end else
        AccessKind := '';
    finally
      DataSet.Free;
    end;
  end;
end;

procedure TAppSecurityDataProvider.AddLogRecord(ADateTime: TDateTime; AUserID:
  Integer; AUserName, AComputerName: String; AMessageType: Integer; AMessage:
  String; AUserData: TObject);
var
  ARecordID: Integer;
  Query: String;
begin
  with DBStructure do
  begin
    Query := Format('INSERT INTO %s (%s, %s, %s, %s, %s, %s) VALUES (%d, ''%s'', ''%s'', ''%s'', %d, ''%s'')',
      [LogTable,

       LogTableUserIDField, LogTableUserNameField, LogTableDateField,
       LogTableComputerNameField, LogTableMessageTypeField, LogTableMessageField,

       AUserID, AUserName, FormatDateTime(DateTimeFormat, ADateTime),
       AComputerName, AMessageType, AMessage]);
    ExecuteQuery(Query);

    if Assigned(AUserData) then
    begin
      ARecordID := GetMaxValue(LogTable, LogTableIDField);
      AddLogData(ARecordID, AUserData);
    end;
  end;
end;

{{
       0,  
.
}
function TAppSecurityDataProvider.GetLastLogRecordID: Integer;
begin
  with DBStructure do
    Result := GetMaxValue(LogTable, LogTableIDField);
end;

{{
   ( )    
         0,    .
}
function TAppSecurityDataProvider.GetLastLogRecordID(ALogRecordType: Integer;
  ALogRecordMessage: String): Integer;
var
  Query: String;
begin
  with DBStructure do
    Query := Format('SELECT TOP 1 %s FROM %s WHERE %s = %d AND %s = ''%s'' ORDER BY %s DESC',
      [LogTableIDField, LogTable, LogTableMessageTypeField, ALogRecordType,
       LogTableMessageField, ALogRecordMessage, LogTableDateField]);
  with OpenQuery(Query) do
  try
    if Not IsEmpty then
      Result := Fields[0].AsInteger
    else
      Result := 0
  finally
    Free
  end;
end;

end.
