unit asfLogViewer;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, ComCtrls, asfClasses;

type
  TfmLogViewer = class(TForm)
    RecordView: TListView;
    Panel1: TPanel;
    Label1: TLabel;
    edStartDate: TDateTimePicker;
    Label2: TLabel;
    edEndDate: TDateTimePicker;
    UpdateButton: TButton;
    Bevel1: TBevel;
    CloseButton: TButton;
    EraseButton: TButton;
    procedure EraseButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure RecordViewDblClick(Sender: TObject);
    procedure RecordViewKeyPress(Sender: TObject; var Key: Char);
    procedure UpdateButtonClick(Sender: TObject);
  private
    FAppSecurity: TAppSecurity;
    FRecordIDList: array of Integer;
    FRecordTypeList: array of Integer;
    { Private declarations }
  protected
    procedure ShowRecordData;
  public
    procedure ShowRecords;
    { Public declarations }
  end;

procedure ShowSecurityLog(AppSecurity: TAppSecurity);

implementation

uses
  DateUtils, DB;

{$R *.dfm}

procedure ShowSecurityLog(AppSecurity: TAppSecurity);
begin
  with TfmLogViewer.Create(Application) do
  try
    FAppSecurity := AppSecurity;
    ShowModal
  finally
    Free
  end;
end;

{ TfmLogViewer }

procedure TfmLogViewer.FormCreate(Sender: TObject);
begin
  edStartDate.Date := IncDay(Date, -10);
  edEndDate.Date := Date;
end;

procedure TfmLogViewer.FormShow(Sender: TObject);
begin
  EraseButton.Enabled := FAppSecurity.LogOptions.AllowDeleteRecords;

  ShowRecords
end;

procedure TfmLogViewer.UpdateButtonClick(Sender: TObject);
begin
  ShowRecords
end;

procedure TfmLogViewer.ShowRecords;

  procedure LoadRecord(ADataSet: TDataSet);
  var
    ListItem: TListItem;
  begin
    ListItem := RecordView.Items.Add;
    with FAppSecurity.DataProvider.DBStructure do
    begin
      FRecordIDList[ListItem.Index] := ADataSet.FieldByName(LogTableIDField).AsInteger;
      FRecordTypeList[ListItem.Index] := ADataSet.FieldByName(LogTableMessageTypeField).AsInteger;

      ListItem.Caption := ADataSet.FieldByName(LogTableDateField).AsString;
      ListItem.SubItems.Add(ADataSet.FieldByName(LogTableUserNameField).AsString);
      ListItem.SubItems.Add(ADataSet.FieldByName(LogTableComputerNameField).AsString);
      ListItem.SubItems.Add(ADataSet.FieldByName(LogTableMessageField).AsString);
    end;
  end;

var
  ADataSet: TDataSet;
begin
  ADataSet := FAppSecurity.DataProvider.GetLogRecords(edStartDate.Date, IncDay(edEndDate.Date));
  try
    SetLength(FRecordIDList, ADataSet.RecordCount);
    SetLength(FRecordTypeList, ADataSet.RecordCount);

    with RecordView.Items do
    begin
      BeginUpdate;
      try
        Clear;
        with ADataSet do
        begin
          First;
          while Not Eof do
          begin
            LoadRecord(ADataSet);
            Next;
          end
        end;
      finally
        EndUpdate;
      end;
    end;
  finally
    ADataSet.Free;
  end;
end;

procedure TfmLogViewer.EraseButtonClick(Sender: TObject);
begin
  if Application.MessageBox(PChar(Format('    %s  %s ?',
    [DateToStr(edStartDate.Date), DateToStr(edEndDate.Date)])), ' ',
    MB_ICONQUESTION or MB_YESNO) = idYes then
  begin
    FAppSecurity.DataProvider.DeleteLogRecords(edStartDate.Date, IncDay(edEndDate.Date));
    ShowRecords;
  end;
end;

procedure TfmLogViewer.RecordViewDblClick(Sender: TObject);
begin
  ShowRecordData
end;

procedure TfmLogViewer.RecordViewKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #13 then
  begin
    ShowRecordData;
    Key := #0;
  end;
end;

procedure TfmLogViewer.ShowRecordData;
begin
  with FAppSecurity do
    if Assigned(OnShowLogRecordData) and Assigned(RecordView.Selected) then
      OnShowLogRecordData(FAppSecurity,
        FRecordIDList[RecordView.Selected.Index], FRecordTypeList[RecordView.Selected.Index])
end;

end.
