unit asfLoginDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, asfClasses, ComCtrls;

type
  TfmLoginDialog = class(TForm)
    Image1: TImage;
    Bevel: TBevel;
    FullNameLabel: TLabel;
    LoginLabel: TLabel;
    PasswordLabel: TLabel;
    edLogin: TEdit;
    edPassword: TEdit;
    OkButton: TButton;
    CancelButton: TButton;
    cbFullName: TComboBox;
    StatusBar: TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure cbFullNameSelect(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    FAppSecurity: TAppSecurity;
    FUserID: Integer;
    FUserName: String;
    FUserLogin: String;
    FTryCount: Integer;
    procedure FillUserNames;
    function AcceptCredentials: Boolean;
  public
    { Public declarations }
    property UserID: Integer read FUserID;
    property UserName: String read FUserName;
    property UserLogin: String read FUserLogin;
  end;

  TLoginResult = (lrSuccess, lrCancel, lrFailure);

function ShowLoginDialog(AppSecurity: TAppSecurity; var AppUser:
  TApplicationUser; AHint: String = ''): TLoginResult;

implementation

{$R *.dfm}

{{
    .    AppSecurity:
MaxLoginTryCount, AllowSelectUser.   
  TApplicationUser   .
}
function ShowLoginDialog(AppSecurity: TAppSecurity; var AppUser:
  TApplicationUser; AHint: String = ''): TLoginResult;
begin
  with TfmLoginDialog.Create(Application) do
  try
    FAppSecurity := AppSecurity;
    AppUser := nil;
    StatusBar.SimpleText := AHint;
    if ShowModal = mrOk then
    begin
      if UserID <> 0 then
      begin
        AppUser := TApplicationUser.Create(UserID, UserLogin, UserName);
        Result := lrSuccess;
      end else
        Result := lrFailure;
    end else
      Result := lrCancel;
  finally
    Free
  end;
end;

{ TfmLoginDialog }

procedure TfmLoginDialog.FormCreate(Sender: TObject);
begin

end;

{{
        .
}
procedure TfmLoginDialog.FormShow(Sender: TObject);
var
  AllowSelectUser: Boolean;
  H: Integer;
begin
  if StatusBar.SimpleText = '' then
  begin
    StatusBar.Hide;
    Height := Height - StatusBar.Height;
  end;

  AllowSelectUser := soAllowSelectUserInLogin in FAppSecurity.Options;

  cbFullName.Visible := AllowSelectUser;
  FullNameLabel.Visible := AllowSelectUser;
  if AllowSelectUser then FillUserNames
  else begin
    H := cbFullName.Top + cbFullName.Height - FullNameLabel.Top;
    LoginLabel.Top := LoginLabel.Top - H;
    edLogin.Top := edLogin.Top - H;
    PasswordLabel.Top := PasswordLabel.Top - H;
    edPassword.Top := edPassword.Top - H;
    Bevel.Height := Bevel.Height - H;
    OkButton.Top := OkButton.Top - H;
    CancelButton.Top := CancelButton.Top - H;
    Height := Height - H;
  end;

  FTryCount := 0;
end;

procedure TfmLoginDialog.FillUserNames;
begin
  {       }
  FAppSecurity.DataProvider.GetUsers(cbFullName.Items, False);
end;

{{
          
}
procedure TfmLoginDialog.cbFullNameSelect(Sender: TObject);
var
  UserID: Integer;
  AUserName, ALogin, APassword: String;
  ARoleID: Integer;
begin
  UserID := Integer(cbFullName.Items.Objects[cbFullName.ItemIndex]);
  FAppSecurity.DataProvider.GetUserInfo(UserID, AUserName, ALogin, APassword, ARoleID);
  edLogin.Text := ALogin;
  edPassword.SetFocus;
end;

{{
    .     
LoginNumberOfTries-1 .
}
procedure TfmLoginDialog.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if ModalResult = mrOk then
  begin
    if AcceptCredentials then CanClose := True
    else begin
      Application.MessageBox('     ',
        ' ', MB_ICONERROR or MB_OK);
      if FAppSecurity.LoginNumberOfTries = 0 then
        CanClose := False
      else
      if FTryCount < FAppSecurity.LoginNumberOfTries-1 then
      begin
        Inc(FTryCount);
        CanClose := False;
      end else
      begin
        Application.MessageBox('    ',
          ' ', MB_ICONERROR or MB_OK);
        CanClose := True;
        FUserID := 0;
      end;
    end;
  end;
end;

function TfmLoginDialog.AcceptCredentials: Boolean;
var
  I: Integer;
  Users: TStrings;
  Password: String;
  RoleID: Integer;
begin
  Users := TStringList.Create;
  try
    FAppSecurity.DataProvider.GetUsers(Users, True);
    I := Users.IndexOf(edLogin.Text);
    if I < 0 then Result := False
    else begin
      FUserID := Integer(Users.Objects[I]);
      FAppSecurity.DataProvider.GetUserInfo(UserID, FUserName, FUserLogin, Password, RoleID);
      Result := edPassword.Text = Password
    end;
  finally
    Users.Free;
  end;
end;

end.
