unit asfModel;

interface

uses SysUtils, Classes, Contnrs;

type
  TAppSecurityModel = class;

  TAppSecurityModelItems = class(TCollection)
  private
    FModel: TAppSecurityModel;
  public
    constructor Create(AModel: TAppSecurityModel);
    property Model: TAppSecurityModel read FModel;
  end;

  TAppSecurityModelItem = class(TCollectionItem)
  private
    FParentItem: TAppSecurityModelItem;
    FAccessKinds: TStrings;
    FID: Integer;
    FItems: TAppSecurityModelItems;
    FName: String;
    FModel: TAppSecurityModel;
    procedure SetAccessKinds(Value: TStrings);
    function GetChildItemCount: Integer;
    function GetChildItems(Index: Integer): TAppSecurityModelItem;
    procedure SetParentItem(const Value: TAppSecurityModelItem);
    procedure ReadItems(Reader: TReader);
    procedure WriteItems(Writer: TWriter);
    function FindItemByID(SecurityItemID: Integer): TAppSecurityModelItem;
  protected
    procedure DefineProperties(Filer: TFiler); override;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function HasState(StateName: String): Boolean;
    property ParentItem: TAppSecurityModelItem read FParentItem write SetParentItem;
    {{
      
    }
    property ChildItemCount: Integer read GetChildItemCount;
    {{
         
    }
    property ChildItems[Index: Integer]: TAppSecurityModelItem read GetChildItems;
  published
    property Name: String read FName write FName;
    {{
      .
    }
    property ID: Integer read FID write FID;
    {{
        .
    }
    property AccessKinds: TStrings read FAccessKinds write SetAccessKinds;
  end;

  TAppSecurityModelEnumerator = class
  private
    FModel: TAppSecurityModel;
    FItemList: TList;
    FIndex: Integer;
    function GetCurrent: TAppSecurityModelItem;
    procedure GenerateItemList;
  public
    constructor Create(AModel: TAPpSecurityModel);
    function MoveNext: Boolean;
    destructor Destroy; override;
    property Current: TAppSecurityModelItem read GetCurrent;
    property Model: TAppSecurityModel read FModel;
  end;

  TAppSecurityModel = class(TComponent)
  private
    FItems: TAppSecurityModelItems;
    FNewItemID: Integer;
    FEnabledState: String;
    FDisabledState: String;
    function GetItems(Index: Integer): TAppSecurityModelItem;
    function GetItemCount: Integer;
    function GetNewItemID: Integer;
    procedure ReadItems(Reader: TReader);
    procedure WriteItems(Writer: TWriter);
  protected
    procedure DefineProperties(Filer: TFiler); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Clear;
    {{
       
    }
    function AddItem(AName: String): TAppSecurityModelItem;
    {{
           
    }
    procedure DeleteItem(AItem: TAppSecurityModelItem);
    procedure Assign(Source: TPersistent); override;
    function GetItemByID(SecurityItemID: Integer): TAppSecurityModelItem;
    function FindItemByID(SecurityItemID: Integer): TAppSecurityModelItem;
    {{
         
    }
    property Items[Index: Integer]: TAppSecurityModelItem read GetItems;
    {{
      
    }
    property ItemCount: Integer read GetItemCount;
    function GetEnumerator: TAppSecurityModelEnumerator;
  published
    property NewItemID: Integer read FNewItemID write FNewItemID;
    property EnabledState: String read FEnabledState write FEnabledState;
    property DisabledState: String read FDisabledState write FDisabledState;
  end;

implementation

{ TAppSecurityModelItems }

constructor TAppSecurityModelItems.Create(AModel: TAppSecurityModel);
begin
  inherited Create(TAppSecurityModelItem);
  FModel := AModel;
end;

{ TAppSecurityModel }

constructor TAppSecurityModel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FItems := TAppSecurityModelItems.Create(Self);
  FEnabledState := '';
  FDisabledState := '';
end;

destructor TAppSecurityModel.Destroy;
begin
  FreeAndNil(FItems);
  inherited Destroy;
end;

procedure TAppSecurityModel.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty('Items', ReadItems, WriteItems, FItems.Count > 0);
end;

procedure TAppSecurityModel.ReadItems(Reader: TReader);
begin
  Reader.ReadValue;
  Reader.ReadCollection(FItems)
end;

procedure TAppSecurityModel.WriteItems(Writer: TWriter);
begin
  Writer.WriteCollection(FItems)
end;

function TAppSecurityModel.GetItems(Index: Integer): TAppSecurityModelItem;
begin
  Result := TAppSecurityModelItem(FItems.Items[Index])
end;

function TAppSecurityModel.GetItemCount: Integer;
begin
  Result := FItems.Count
end;

procedure TAppSecurityModel.Clear;
begin
  FItems.Clear;
  FNewItemID := 0;
end;

function TAppSecurityModel.AddItem(AName: String): TAppSecurityModelItem;
begin
  Result := TAppSecurityModelItem(FItems.Add);
  Result.FModel := Self;
  Result.Name := AName;
  Result.ID := GetNewItemID;
end;

procedure TAppSecurityModel.DeleteItem(AItem: TAppSecurityModelItem);
begin
  AItem.Collection.Delete(AItem.Index);
end;

function TAppSecurityModel.GetNewItemID: Integer;
begin
  Inc(FNewItemID);
  Result := NewItemID;
end;

function TAppSecurityModel.FindItemByID(SecurityItemID: Integer):
  TAppSecurityModelItem;
var
  I: Integer;
begin
  for I := 0 to ItemCount - 1 do
  begin
    Result := Items[I].FindItemByID(SecurityItemID);
    if Result <> nil then Exit;
  end;
  Result := nil
end;

function TAppSecurityModel.GetItemByID(SecurityItemID: Integer):
  TAppSecurityModelItem;
begin
  Result := FindItemByID(SecurityItemID);
  if Not Assigned(Result) then
    raise Exception.CreateFmt('   %d  ' , [SecurityItemID]);
end;

function TAppSecurityModel.GetEnumerator: TAppSecurityModelEnumerator;
begin
  Result := TAppSecurityModelEnumerator.Create(Self);
end;

{{
    Source.
}
procedure TAppSecurityModel.Assign(Source: TPersistent);
begin
  if Source is TAppSecurityModel then
  begin
    with TAppSecurityModel(Source) do
    begin
      Self.NewItemID := NewItemID;
      Self.FItems.Assign(FItems);
    end;
  end else
    inherited Assign(Source);
end;

{ TAppSecurityModelItem }

constructor TAppSecurityModelItem.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FModel := (Collection as TAppSecurityModelItems).Model;
  FItems := TAppSecurityModelItems.Create(FModel);

  FAccessKinds := TStringList.Create;
  FAccessKinds.Add(FModel.EnabledState);
  FAccessKinds.Add(FModel.DisabledState);
end;

destructor TAppSecurityModelItem.Destroy;
begin
  FreeAndNil(FAccessKinds);
  FreeAndNil(FItems);
  inherited Destroy;
end;

procedure TAppSecurityModelItem.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty('Items', ReadItems, WriteItems, FItems.Count > 0);
end;

procedure TAppSecurityModelItem.ReadItems(Reader: TReader);
begin
  Reader.ReadValue;
  Reader.ReadCollection(FItems)
end;

procedure TAppSecurityModelItem.WriteItems(Writer: TWriter);
begin
  Writer.WriteCollection(FItems)
end;

function TAppSecurityModelItem.GetChildItemCount: Integer;
begin
  Result := FItems.Count
end;

function TAppSecurityModelItem.GetChildItems(Index: Integer):
  TAppSecurityModelItem;
begin
  Result := TAppSecurityModelItem(FItems.Items[Index])
end;

procedure TAppSecurityModelItem.SetAccessKinds(Value: TStrings);
begin
  FAccessKinds.Assign(Value);
end;

function TAppSecurityModelItem.HasState(StateName: String): Boolean;
begin
  Result := AccessKinds.IndexOf(StateName) >= 0
end;

procedure TAppSecurityModelItem.SetParentItem(const Value:
  TAppSecurityModelItem);
begin
  if FParentItem <> Value then
  begin
    FParentItem := Value;
    if Assigned(Value) then
      Collection := FParentItem.FItems
    else
      Collection := FModel.FItems
  end;
end;

{{
   Source   .
}
procedure TAppSecurityModelItem.Assign(Source: TPersistent);
begin
  if Source is TAppSecurityModelItem then
  begin
    with TAppSecurityModelItem(Source) do
    begin
      {     }
      Self.Name := Name;
      Self.ID := ID;
      Self.AccessKinds := AccessKinds;

      {    }
      Self.FItems.Assign(FItems);
    end;
  end else
    inherited Assign(Source);
end;

function TAppSecurityModelItem.FindItemByID(SecurityItemID: Integer):
  TAppSecurityModelItem;
var
  I: Integer;
begin
  if SecurityItemID = ID then Result := Self
  else begin
    for I := 0 to ChildItemCount - 1 do
    begin
      Result := ChildItems[I].FindItemByID(SecurityItemID);
      if Result <> nil then Exit;
    end;
    Result := nil
  end;
end;

{ TAppSecurityModelEnumerator }

constructor TAppSecurityModelEnumerator.Create(AModel: TAPpSecurityModel);
begin
  inherited Create;
  FModel := AModel;
  FItemList := TList.Create;
  GenerateItemList;
  FIndex := -1;
end;

destructor TAppSecurityModelEnumerator.Destroy;
begin
  FreeAndNil(FItemList);
  inherited Destroy;
end;

procedure TAppSecurityModelEnumerator.GenerateItemList;

  procedure AddItems(ModelItem: TAppSecurityModelItem);
  var
    ChildItem: TObject;
  begin
    FItemList.Add(ModelItem);
    for ChildItem in ModelItem.FItems do
      AddItems(TAppSecurityModelItem(ChildItem))
  end;

var
  Item: TObject;
begin
  FItemList.Clear;
  for Item in Model.FItems do
    AddItems(TAppSecurityModelItem(Item))
end;

function TAppSecurityModelEnumerator.MoveNext: Boolean;
begin
if FIndex < FItemList.Count - 1 then
  begin
    Inc(FIndex);
    Result := True;
  end else
    Result := False;
end;

function TAppSecurityModelEnumerator.GetCurrent: TAppSecurityModelItem;
begin
  Result := TAppSecurityModelItem(FItemList[FIndex])
end;

end.
