unit asfModelTreeView;

interface

uses Windows, SysUtils, Classes, Controls, CommCtrl, ComCtrls, Dialogs, asfModel;

type
  {{
   TAppSecurityModelTreeView   
  TAppSecurityModel,   
  -   
  -  
  -  
  -    .
  }
  TAppSecurityModelTreeView = class(TTreeView)
  private
    FModel: TAppSecurityModel;
    FConfirmDelete: Boolean;
    FImageIndex: Integer;
    procedure BuildItems(AParentNode: TTreeNode; AModelItem: TAppSecurityModelItem);
    procedure CheckModel;
    procedure SetModel(const Value: TAppSecurityModel);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure Edit(const Item: TTVItem); override;
    procedure DragOver(Source: TObject; X, Y: Integer; State: TDragState; var
      Accept: Boolean); override;
  public
    procedure InsertNewItem;
    procedure DeleteCurrentItem;
    procedure RefreshTreeView;
    constructor Create(AOwner: TComponent); override;
    procedure DragDrop(Source: TObject; X, Y: Integer); override;
    procedure Clear;
    function GetModelItemFromNode(Node: TTreeNode): TAppSecurityModelItem;
  published
    property Model: TAppSecurityModel read FModel write SetModel;
    property ConfirmDelete: Boolean read FConfirmDelete write FConfirmDelete
      default True;
    property ImageIndex: Integer read FImageIndex write FImageIndex default 0;
  end;

implementation

resourcestring
  SNewItemName = ' ';

{ TAppSecurityModelTreeView }

constructor TAppSecurityModelTreeView.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FConfirmDelete := True;
  DragMode := dmAutomatic;
end;

procedure TAppSecurityModelTreeView.SetModel(const Value: TAppSecurityModel);
begin
  if FModel <> Value then
  begin
    FModel := Value;
    if Value <> nil then
    begin
      Value.FreeNotification(Self);
      RefreshTreeView;
    end;
  end;
end;

procedure TAppSecurityModelTreeView.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
    if AComponent = Model then Model := nil
end;

procedure TAppSecurityModelTreeView.CheckModel;
begin
  if Not Assigned(Model) then raise Exception.Create('   Model');
end;

function TAppSecurityModelTreeView.GetModelItemFromNode(Node: TTreeNode):
  TAppSecurityModelItem;
begin
  Result := TAppSecurityModelItem(Node.Data);
end;

{***************************************************************************** }
{                                                    }
{ **************************************************************************** }

procedure TAppSecurityModelTreeView.RefreshTreeView;
var
  I: Integer;
begin
  Items.BeginUpdate;
  try
    Items.Clear;
    with Model do
      for I := 0 to ItemCount - 1 do BuildItems(nil, Items[I]);
  finally
    Items.EndUpdate;
  end;
end;

procedure TAppSecurityModelTreeView.BuildItems(AParentNode: TTreeNode;
  AModelItem: TAppSecurityModelItem);
var
  Node: TTreeNode;
  I: Integer;
begin
  Node := Items.AddChild(AParentNode, AModelItem.Name);
  Node.ImageIndex := ImageIndex;
  Node.SelectedIndex := ImageIndex;
  Node.Data := AModelItem;

  for I := 0 to AModelItem.ChildItemCount - 1 do
    BuildItems(Node, AModelItem.ChildItems[I]);
end;

{***************************************************************************** }
{                                                 }
{ **************************************************************************** }

procedure TAppSecurityModelTreeView.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited;
  if Key = VK_INSERT then InsertNewItem else
  if Key = VK_DELETE then DeleteCurrentItem
  else Exit;
  Key := 0;
end;

procedure TAppSecurityModelTreeView.InsertNewItem;
var
  ANewItem: TAppSecurityModelItem;
  Node: TTreeNode;
begin
  CheckModel;
  ANewItem := Model.AddItem(SNewItemName);
  Node := Items.Add(nil, ANewItem.Name);
  Node.ImageIndex := ImageIndex;
  Node.SelectedIndex := ImageIndex;
  Node.Data := ANewItem;
  Node.Selected := True;
end;

procedure TAppSecurityModelTreeView.DeleteCurrentItem;
begin
  CheckModel;
  if Assigned(Selected) then
  begin
    if Not ConfirmDelete or (MessageDlg('  "' + Selected.Text +'"?',
      mtConfirmation, [mbYes, mbNo], 0) = mrYes) then
    begin
      Model.DeleteItem(TAppSecurityModelItem(Selected.Data));
      Selected.Delete;
    end;
  end;
end;

procedure TAppSecurityModelTreeView.Clear;
begin
  CheckModel;
  Items.Clear;
  Model.Clear;
end;

procedure TAppSecurityModelTreeView.Edit(const Item: TTVItem);
var
  OldText: String;
begin
  if Selected <> nil then OldText := Selected.Text;
  inherited;
  if (Selected <> nil) and (Selected.Text <> OldText) then
    TAppSecurityModelItem(Selected.Data).Name := Selected.Text;
end;

procedure TAppSecurityModelTreeView.DragOver(Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  inherited;
  Accept := true
end;

procedure TAppSecurityModelTreeView.DragDrop(Source: TObject; X, Y: Integer);
var
  Node: TTreeNode;
begin
  if Source = Self then
  begin
    Node := GetNodeAt(X, Y);
    Selected.MoveTo(Node, naAddChild);
    if Node <> nil then
      TAppSecurityModelItem(Selected.Data).ParentItem :=
        TAppSecurityModelItem(Node.Data)
    else
      TAppSecurityModelItem(Selected.Data).ParentItem := nil;
  end;
  inherited;
end;

end.
